## Program: VMTK
## Language:  Python
## Date:      April 9, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkimagevolumeviewer as volumeviewer
import copy


@pytest.fixture(scope='module')
def expected_results_dict(input_datadir):
    from json import load
    import os
    with open(os.path.join(input_datadir, 'vmtkimagevolumeviewer_testresults.json'), 'r') as f:
        resultDict = load(f)
    return resultDict


@pytest.mark.parametrize('preset', [
    ("CT-AAA"), ("CT-AAA2"), ("CT-Bone"),
    ("CT-Bones"), ("CT-Cardiac"), ("CT-Cardiac2"),
    ("CT-Cardiac3"), ("CT-Chest-Contrast-Enhanced"), ("CT-Chest-Vessels"),
    ("CT-Coronary-Arteries"), ("CT-Coronary-Arteries-2"), ("CT-Coronary-Arteries-3"),
    ("CT-Cropped-Volume-Bone"), ("CT-Fat"), ("CT-Liver-Vasculature"),
    ("CT-Lung"), ("CT-MIP"), ("CT-Muscle"), ("CT-Pulmonary-Arteries"),
    ("CT-Soft-Tissue"), ("MR-Angio"), ("MR-Default"),
    ("MR-MIP"), ("MR-T2-Brain")
])
def test_volume_viewer_preset_values(aorta_image, preset, expected_results_dict):
    view = volumeviewer.vmtkImageVolumeViewer()
    view.Image = aorta_image
    view.Display = 0
    view.Preset = preset
    view.Execute()

    interpolationType = view.InterpolationType
    shade = view.Shade
    specularPower = view.SpecularPower
    specular = view.Specular
    diffuse = view.Diffuse
    ambient = view.Ambient

    colorTransferFunctionPointMin = view.ColorTransferFunction.GetRange()[0]
    colorTransferFunctionPointMax = view.ColorTransferFunction.GetRange()[1]
    colorTransferFunctionRangeWidth = colorTransferFunctionPointMax - colorTransferFunctionPointMin
    colorTransferFunctionSamplePointSmall = colorTransferFunctionPointMin + (colorTransferFunctionRangeWidth * 0.1)
    colorTransferFunctionSamplePointLarge = colorTransferFunctionPointMax - (colorTransferFunctionRangeWidth * 0.1)
    colorTransferFunctionColorAtSamplePointSmall = view.ColorTransferFunction.GetColor(colorTransferFunctionSamplePointSmall)
    colorTransferFunctionColorAtSamplePointLarge = view.ColorTransferFunction.GetColor(colorTransferFunctionSamplePointLarge)

    opacityTransferFunctionPointMin = view.OpacityTransferFunction.GetRange()[0]
    opacityTransferFunctionPointMax = view.OpacityTransferFunction.GetRange()[1]
    opacityTransferFunctionRangeWidth = opacityTransferFunctionPointMax - opacityTransferFunctionPointMin
    opacityTransferFunctionSamplePointSmall = opacityTransferFunctionPointMin + (opacityTransferFunctionRangeWidth * 0.1)
    opacityTransferFunctionSamplePointLarge = opacityTransferFunctionPointMax - (opacityTransferFunctionRangeWidth * 0.1)
    opacityTransferFunctionColorAtSamplePointSmall = view.OpacityTransferFunction.GetValue(opacityTransferFunctionSamplePointSmall)
    opacityTransferFunctionColorAtSamplePointLarge = view.OpacityTransferFunction.GetValue(opacityTransferFunctionSamplePointLarge)

    gradientOpacityTransferFunctionPointMin = view.GradientOpacityTransferFunction.GetRange()[0]
    gradientOpacityTransferFunctionPointMax = view.GradientOpacityTransferFunction.GetRange()[1]
    gradientOpacityTransferFunctionRangeWidth = gradientOpacityTransferFunctionPointMax - gradientOpacityTransferFunctionPointMin
    gradientOpacityTransferFunctionSamplePointSmall = gradientOpacityTransferFunctionPointMin + (gradientOpacityTransferFunctionRangeWidth * 0.1)
    gradientOpacityTransferFunctionSamplePointLarge = gradientOpacityTransferFunctionPointMax - (gradientOpacityTransferFunctionRangeWidth * 0.1)
    gradientOpacityTransferFunctionColorAtSamplePointSmall = view.GradientOpacityTransferFunction.GetValue(gradientOpacityTransferFunctionSamplePointSmall)
    gradientOpacityTransferFunctionColorAtSamplePointLarge = view.GradientOpacityTransferFunction.GetValue(gradientOpacityTransferFunctionSamplePointLarge)

    resultsDict = copy.deepcopy(expected_results_dict[preset])

    # The sample points and interpolated transfer-function values are computed in
    # floating point (scaling by 0.1, VTK's piecewise interpolation), so their last
    # bit differs between platforms' math libraries. Compare the numeric results with
    # pytest.approx rather than exact equality; the categorical InterpolationType and
    # Shade stay exact.
    assert interpolationType == resultsDict["InterpolationType"]
    assert shade == resultsDict["Shade"]
    assert specularPower == pytest.approx(resultsDict["SpecularPower"])
    assert specular == pytest.approx(resultsDict["Specular"])
    assert diffuse == pytest.approx(resultsDict["Diffuse"])
    assert ambient == pytest.approx(resultsDict["Ambient"])
    assert colorTransferFunctionPointMin == pytest.approx(resultsDict["ColorTransferFunctionPointMin"])
    assert colorTransferFunctionPointMax == pytest.approx(resultsDict["ColorTransferFunctionPointMax"])
    assert colorTransferFunctionRangeWidth == pytest.approx(resultsDict["ColorTransferFunctionRangeWidth"])
    assert colorTransferFunctionSamplePointSmall == pytest.approx(resultsDict["ColorTransferFunctionSamplePointSmall"])
    assert colorTransferFunctionSamplePointLarge == pytest.approx(resultsDict["ColorTransferFunctionSamplePointLarge"])
    assert colorTransferFunctionColorAtSamplePointSmall == pytest.approx(tuple(resultsDict["ColorTransferFunctionColorAtSamplePointSmall"]))
    assert colorTransferFunctionColorAtSamplePointLarge == pytest.approx(tuple(resultsDict["ColorTransferFunctionColorAtSamplePointLarge"]))
    assert opacityTransferFunctionPointMin == pytest.approx(resultsDict["OpacityTransferFunctionPointMin"])
    assert opacityTransferFunctionPointMax == pytest.approx(resultsDict["OpacityTransferFunctionPointMax"])
    assert opacityTransferFunctionRangeWidth == pytest.approx(resultsDict["OpacityTransferFunctionRangeWidth"])
    assert opacityTransferFunctionSamplePointSmall == pytest.approx(resultsDict["OpacityTransferFunctionSamplePointSmall"])
    assert opacityTransferFunctionSamplePointLarge == pytest.approx(resultsDict["OpacityTransferFunctionSamplePointLarge"])
    assert opacityTransferFunctionColorAtSamplePointSmall == pytest.approx(resultsDict["OpacityTransferFunctionColorAtSamplePointSmall"])
    assert opacityTransferFunctionColorAtSamplePointLarge == pytest.approx(resultsDict["OpacityTransferFunctionColorAtSamplePointLarge"])
    assert gradientOpacityTransferFunctionPointMin == pytest.approx(resultsDict["GradientOpacityTransferFunctionPointMin"])
    assert gradientOpacityTransferFunctionPointMax == pytest.approx(resultsDict["GradientOpacityTransferFunctionPointMax"])
    assert gradientOpacityTransferFunctionRangeWidth == pytest.approx(resultsDict["GradientOpacityTransferFunctionRangeWidth"])
    assert gradientOpacityTransferFunctionSamplePointSmall == pytest.approx(resultsDict["GradientOpacityTransferFunctionSamplePointSmall"])
    assert gradientOpacityTransferFunctionSamplePointLarge == pytest.approx(resultsDict["GradientOpacityTransferFunctionSamplePointLarge"])
    assert gradientOpacityTransferFunctionColorAtSamplePointSmall == pytest.approx(resultsDict["GradientOpacityTransferFunctionColorAtSamplePointSmall"])
    assert gradientOpacityTransferFunctionColorAtSamplePointLarge == pytest.approx(resultsDict["GradientOpacityTransferFunctionColorAtSamplePointLarge"])



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtksurfacescaling as scaling


def test_isotropic_scale(aorta_surface, compare_surfaces):
    name = __name__ + '_test_isotropic_scale.vtp'
    scaler = scaling.vmtkSurfaceScaling()
    scaler.Surface = aorta_surface
    scaler.ScaleFactor = 2
    scaler.Execute()

    assert compare_surfaces(scaler.Surface, name, tolerance=1.0) == True


@pytest.mark.parametrize('xfactor,yfactor,zfactor,paramid', [
    (2, None, None, '0'),
    (None, 2, None, '1'),
    (None, None, 2, '2'),
    (2, 2, None, '3'),
    (2, None, 2, '4'),
    (None, 2, 2, '5'),
])
def test_xyz_scale_factors(aorta_surface, compare_surfaces, xfactor,
                           yfactor, zfactor, paramid):
    name = __name__ + '_test_xyz_scale_factors_' + paramid + '.vtp'
    scaler = scaling.vmtkSurfaceScaling()
    scaler.Surface = aorta_surface
    scaler.ScaleFactorX = xfactor
    scaler.ScaleFactorY = yfactor
    scaler.ScaleFactorZ = zfactor
    scaler.Execute()

    assert compare_surfaces(scaler.Surface, name, tolerance=1.0) == True



## Program: VMTK
## Language:  Python
## Date:      January 24, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkcenterlines as vmtkcenterlines
import vmtk.vmtkcenterlineviewer as viewer


def test_idlist_centerlines(aorta_surface, compare_centerlines):
    name = __name__ + '_test_idlist_centerlines.vtp'
    centerliner = vmtkcenterlines.vmtkCenterlines()
    centerliner.Surface = aorta_surface
    centerliner.SeedSelectorName = 'idlist'
    centerliner.SourceIds = [2334]
    centerliner.TargetIds = [5561, 6131]
    centerliner.Execute()

    assert compare_centerlines(centerliner.Centerlines, name) == True


def test_carotidprofiles_centerlines(aorta_surface_openends, compare_centerlines):
    name = __name__ + '_test_carotidprofiles_centerlines.vtp'
    centerliner = vmtkcenterlines.vmtkCenterlines()
    centerliner.Surface = aorta_surface_openends
    centerliner.SeedSelectorName = 'carotidprofiles'
    centerliner.Execute()

    assert compare_centerlines(centerliner.Centerlines, name) == True


def test_profileidlist_centerlines(aorta_surface_openends, compare_centerlines):
    name = __name__ + '_test_profileidlist_centerlines.vtp'
    centerliner = vmtkcenterlines.vmtkCenterlines()
    centerliner.Surface = aorta_surface_openends
    centerliner.SeedSelectorName = 'profileidlist'
    centerliner.SourceIds = [0]
    centerliner.TargetIds = [1, 2]
    centerliner.Execute()

    assert compare_centerlines(centerliner.Centerlines, name) == True


def test_profileidlist_centerlines_smooth(aorta_surface_openends, compare_centerlines):
    # centerlines don't change but voronoi diagram does
    # this test verifies that simplification runs without
    # seg fault, but is not evaluated for diagram
    # correctness.
    name = __name__ + '_test_profileidlist_centerlines.vtp'
    centerliner = vmtkcenterlines.vmtkCenterlines()
    centerliner.Surface = aorta_surface_openends
    centerliner.SeedSelectorName = 'profileidlist'
    centerliner.SourceIds = [0]
    centerliner.TargetIds = [1, 2]
    centerliner.SimplifyVoronoi = True
    centerliner.Execute()

    assert compare_centerlines(centerliner.Centerlines, name) == True



## Program: VMTK
## Language:  Python
## Date:      February 12, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkbifurcationprofiles as bifurcationprofiles
from vtk.numpy_interface import dataset_adapter as dsa
import numpy as np


@pytest.fixture(scope='module')
def bifur_profiles(aorta_centerline_branches ,aorta_surface_branches):
    profiles = bifurcationprofiles.vmtkBifurcationProfiles()
    profiles.Surface = aorta_surface_branches
    profiles.Centerlines = aorta_centerline_branches
    profiles.RadiusArrayName = 'MaximumInscribedSphereRadius'
    profiles.GroupIdsArrayName = 'GroupIds'
    profiles.CenterlineIdsArrayName = 'CenterlineIds'
    profiles.TractIdsArrayName = 'TractIds'
    profiles.BlankingArrayName = 'Blanking'
    profiles.Execute()

    return profiles.BifurcationProfiles


@pytest.mark.parametrize("expectedKey",[
    ('BifurcationProfileGroupIds'),
    ('BifurcationProfileBifurcationGroupIds'),
    ('BifurcationProfileOrientation')
 ])
def test_expected_cell_data_keys_present(bifur_profiles, expectedKey):
    wrapped_bifur_profile = dsa.WrapDataObject(bifur_profiles)
    assert expectedKey in wrapped_bifur_profile.CellData.keys()


def test_number_expected_cell_data_keys(bifur_profiles):
    wrapped_bifur_profile = dsa.WrapDataObject(bifur_profiles)
    assert len(wrapped_bifur_profile.CellData.keys()) == 3


def test_number_expected_point_data_keys(bifur_profiles):
    wrapped_bifur_profile = dsa.WrapDataObject(bifur_profiles)
    assert len(wrapped_bifur_profile.PointData.keys()) == 0


def test_expected_number_points(bifur_profiles):
    wrapped_bifur_profile = dsa.WrapDataObject(bifur_profiles)
    assert wrapped_bifur_profile.Points.shape == (358, 3)


def test_bifurcation_profile_group_ids_correct(bifur_profiles):
    wrapped_bifur_profile = dsa.WrapDataObject(bifur_profiles)
    expectedValue = np.array([0, 2, 3])
    assert np.allclose(wrapped_bifur_profile.CellData.GetArray('BifurcationProfileGroupIds'), expectedValue) == True


def test_bifurcation_profile_bifurcation_group_ids_correct(bifur_profiles):
    wrapped_bifur_profile = dsa.WrapDataObject(bifur_profiles)
    expectedValue = np.array([1, 1, 1])
    assert np.allclose(wrapped_bifur_profile.CellData.GetArray('BifurcationProfileBifurcationGroupIds'), expectedValue) == True


def test_bifurcation_profile_orientation_correct(bifur_profiles):
    wrapped_bifur_profile = dsa.WrapDataObject(bifur_profiles)
    expectedValue = np.array([0, 1, 1])
    assert np.allclose(wrapped_bifur_profile.CellData.GetArray('BifurcationProfileOrientation'), expectedValue) == True


def test_number_of_lines(bifur_profiles):
    assert bifur_profiles.GetNumberOfLines() == 3


def test_number_of_cells(bifur_profiles):
    assert bifur_profiles.GetNumberOfCells() == 3


def test_number_of_points(bifur_profiles):
    assert bifur_profiles.GetNumberOfPoints() == 358


@pytest.mark.parametrize("expectedvalue,paramid", [
    (132, 0),
    (106, 1),
    (120, 2)
])
def test_number_of_points_per_cell(bifur_profiles, expectedvalue, paramid):
    bcx = bifur_profiles.GetCell(paramid)
    numberOfPoints = bcx.GetNumberOfPoints()

    assert numberOfPoints == expectedvalue


@pytest.mark.parametrize("expectedvalue,paramid", [
    (0, 0),
    (132, 1),
    (238, 2)
])
def test_cell_data_pointId_start_indices(bifur_profiles, expectedvalue, paramid):
    bcx = bifur_profiles.GetCell(paramid)
    pointIdStart = bcx.GetPointId(0)

    assert pointIdStart == expectedvalue


@pytest.mark.parametrize("expectedvalue,numberofpoints,paramid", [
    (131, 132, 0),
    (237, 106, 1),
    (357, 120, 2)
])
def test_cell_data_pointId_end_indices(bifur_profiles, expectedvalue, numberofpoints, paramid):
    bcx = bifur_profiles.GetCell(paramid)
    pointIdEnd = bcx.GetPointId(numberofpoints - 1)

    assert pointIdEnd == expectedvalue


@pytest.mark.parametrize("pointidstart,numberofpoints,expectedlocationstart,expectedlocationend,paramid", [
    (0, 132, np.array([ 221.89123535, 135.91229248,  17.98954964]), np.array([ 221.82603455, 135.88433838,  18.00107956]), 0),
    (132, 106, np.array([ 222.36294556, 136.11395264,  17.92448616]), np.array([ 222.40290833, 136.13102722,  17.91927338]), 1),
    (238, 120, np.array([ 221.05075073, 132.63262939,  19.5011692 ]), np.array([ 221.05207825, 132.61680603,  19.50825119]), 2)
])
def test_cell_data_point_start_and_end_xyz_locations(bifur_profiles, pointidstart, numberofpoints,
                                                     expectedlocationstart, expectedlocationend, paramid):
    bcx = bifur_profiles.GetCell(paramid)
    bw = dsa.WrapDataObject(bifur_profiles)

    pointIdEnd = bcx.GetPointId(numberofpoints - 1)
    pointLocationEnd = bw.Points[pointIdEnd]
    pointLocationStart = bw.Points[pointidstart]

    assert np.allclose(np.array(pointLocationStart), expectedlocationstart) == True
    assert np.allclose(np.array(pointLocationEnd), expectedlocationend) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimageobjectenhancement.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes
from vmtk import vtkvmtk


class vmtkImageObjectEnhancement(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None
        self.ScalesImage = None
        self.EnhancedImage = None

        self.SigmaMin = 1.0
        self.SigmaMax = 1.0
        self.NumberOfSigmaSteps = 1

        self.ScaledObjectness = 0

        self.Alpha = 0.5
        self.Beta = 0.5
        self.Gamma = 5.0

        self.ObjectDimension = 0

        self.SetScriptName('vmtkimageobjectenhancement')
        self.SetScriptDoc('compute a feature image for use in segmentation')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['SigmaMin','sigmamin','float',1,'(0.0,)'],
            ['SigmaMax','sigmamax','float',1,'(0.0,)'],
            ['NumberOfSigmaSteps','sigmasteps','int',1,'(0,)'],
            ['ScaledObjectness','scaled','bool',1,''],
            ['Alpha','alpha','float',1,'(0.0,)',''],
            ['Beta','beta','float',1,'(0.0,)',''],
            ['Gamma','gamma','float',1,'(0.0,)',''],
            ['ObjectDimension','dimension','int',1,'(0,2)','']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter'],
            ['ScalesImage','oscales','vtkImageData',1,'','the scales image','vmtkimagewriter']
            ])

    def ApplyObjectness(self):

        objectness = vtkvmtk.vtkvmtkObjectnessMeasureImageFilter()
        objectness.SetInputData(self.Image)
        objectness.SetSigmaMin(self.SigmaMin)
        objectness.SetSigmaMax(self.SigmaMax)
        objectness.SetNumberOfSigmaSteps(self.NumberOfSigmaSteps)
        objectness.SetAlpha(self.Alpha)
        objectness.SetBeta(self.Beta)
        objectness.SetGamma(self.Gamma)
        objectness.SetObjectDimension(self.ObjectDimension)
        objectness.Update()

        self.EnhancedImage = vtk.vtkImageData()
        self.EnhancedImage.DeepCopy(objectness.GetOutput())

        self.ScalesImage = vtk.vtkImageData()
        self.ScalesImage.DeepCopy(objectness.GetScalesOutput())

    def Execute(self):

        if self.Image == None:
            self.PrintError('Error: No input image.')

        if self.SigmaMax < self.SigmaMin:
            self.SigmaMax = self.SigmaMin

        self.ApplyObjectness()

        self.Image = self.EnhancedImage


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkimagesmoothing as imagesmoothing


# with default gauss smoothig
def test_default_smoothing(aorta_image, compare_images):
    name = __name__ + '_test_default_smoothing.mha'
    smoother = imagesmoothing.vmtkImageSmoothing()
    smoother.Image = aorta_image
    smoother.Execute()

    assert compare_images(smoother.Image, name) == True

# with default gauss smoothing


def test_smoothing_high_standard_deviation(aorta_image, compare_images):
    name = __name__ + '_test_smoothing_high_standard_deviation.mha'
    smoother = imagesmoothing.vmtkImageSmoothing()
    smoother.Image = aorta_image
    smoother.StandardDeviation = 7.6
    smoother.Execute()

    assert compare_images(smoother.Image, name) == True

# with default gauss smoothing


def test_lower_radius_factor(aorta_image, compare_images):
    name = __name__ + '_test_lower_radius_factor.mha'
    smoother = imagesmoothing.vmtkImageSmoothing()
    smoother.Image = aorta_image
    smoother.RadiusFactor = 3.4
    smoother.Execute()

    assert compare_images(smoother.Image, name) == True


def test_anisotropic_smoothing_default_params(aorta_image, compare_images, write_image):
    name = __name__ + '_test_anisotropic_smoothing_default_params.mha'
    smoother = imagesmoothing.vmtkImageSmoothing()
    smoother.Image = aorta_image
    smoother.Method = 'anisotropic'
    smoother.Execute()

    write_image(smoother.Image, name)
    assert compare_images(smoother.Image, name) == True


@pytest.mark.parametrize("TimeStep,AutoCalculateTimeStep,Conductance,NumberOfIterations,paramid", [
    (0.0450, 0, 1.0, 5, '0'),
    (0.0450, 1, 1.0, 5, '1'),
    (0.0625, 1, 2.0, 5, '2'),
    (0.0625, 1, 1.0, 3, '3'),
    (0.0625, 1, 2.0, 6, '4'),
])
def test_anisotropic_smoother_varied_params(aorta_image, compare_images,
                                            TimeStep, AutoCalculateTimeStep, Conductance,
                                            NumberOfIterations, paramid, write_image):
    name = __name__ + '_test_anisotropic_smoother_varied_params_' + paramid + '.mha'
    smoother = imagesmoothing.vmtkImageSmoothing()
    smoother.Image = aorta_image
    smoother.Method = 'anisotropic'
    smoother.TimeStep = TimeStep
    smoother.AutoCalculateTimeStep = AutoCalculateTimeStep
    smoother.Conductance = Conductance
    smoother.NumberOfIterations = NumberOfIterations
    smoother.Execute()

    write_image(smoother.Image, name)
    assert compare_images(smoother.Image, name) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkcenterlines.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:52:56 $
## Version:   $Revision: 1.20 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes


## TODO: make SeedSelector a separate pype script to be used in other contexts
class vmtkSeedSelector(object):

    def __init__(self):
        self._Surface = None
        self._SeedIds = None
        self._SourceSeedIds = vtk.vtkIdList()
        self._TargetSeedIds = vtk.vtkIdList()
        self.PrintError = None
        self.PrintLog = None
        self.InputText = None
        self.OutputText = None
        self.InputInfo = None

    def SetSurface(self,surface):
        self._Surface = surface

    def GetSurface(self):
        return self._Surface

    def GetSourceSeedIds(self):
        return self._SourceSeedIds

    def GetTargetSeedIds(self):
        return self._TargetSeedIds

    def Execute(self):
        pass


class vmtkIdListSeedSelector(vmtkSeedSelector):

    def __init__(self):
        vmtkSeedSelector.__init__(self)
        self.SourceIds = None
        self.TargetIds = None

    def Execute(self):

        if not self._Surface:
            self.PrintError('vmtkIdListSeedSelector Error: Surface not set.')
            return

        if not self.SourceIds:
            self.PrintError('vmtkIdListSeedSelector Error: SourceIds not set.')
            return

        if not self.TargetIds:
            self.PrintError('vmtkIdListSeedSelector Error: TargetIds not set.')
            return

        self._SourceSeedIds.Initialize()
        self._TargetSeedIds.Initialize()

        maxId = self._Surface.GetNumberOfPoints()-1

        for id in self.SourceIds:
            if id > maxId:
              self.PrintError('vmtkIdListSeedSelector Error: invalid SourceId.')
            self._SourceSeedIds.InsertNextId(id)

        for id in self.TargetIds:
            if id > maxId:
              self.PrintError('vmtkIdListSeedSelector Error: invalid TargetId.')
            self._TargetSeedIds.InsertNextId(id)


class vmtkPointListSeedSelector(vmtkSeedSelector):

    def __init__(self):
        vmtkSeedSelector.__init__(self)
        self.SourcePoints = None
        self.TargetPoints = None

    def Execute(self):

        if not self._Surface:
            self.PrintError('vmtkPointListSeedSelector Error: Surface not set.')
            return

        if not self.SourcePoints:
            self.PrintError('vmtkPointListSeedSelector Error: SourcePoints not set.')
            return

        if not self.TargetPoints:
            self.PrintError('vmtkPointListSeedSelector Error: TargetPoints not set.')
            return

        self._SourceSeedIds.Initialize()
        self._TargetSeedIds.Initialize()

        if len(self.SourcePoints) % 3 != 0:
            self.PrintError('vmtkPointListSeedSelector Error: SourcePoints not made up of triplets.')
            return

        if len(self.TargetPoints) % 3 != 0:
            self.PrintError('vmtkPointListSeedSelector Error: TargetPoints not made up of triplets.')
            return

        pointLocator = vtk.vtkPointLocator()
        pointLocator.SetDataSet(self._Surface)
        pointLocator.BuildLocator()

        for i in range(len(self.SourcePoints)//3):
            point = [self.SourcePoints[3*i+0],self.SourcePoints[3*i+1],self.SourcePoints[3*i+2]]
            id = pointLocator.FindClosestPoint(point)
            self._SourceSeedIds.InsertNextId(id)

        for i in range(len(self.TargetPoints)//3):
            point = [self.TargetPoints[3*i+0],self.TargetPoints[3*i+1],self.TargetPoints[3*i+2]]
            id = pointLocator.FindClosestPoint(point)
            self._TargetSeedIds.InsertNextId(id)


class vmtkPickPointSeedSelector(vmtkSeedSelector):

    def __init__(self):
        vmtkSeedSelector.__init__(self)
        self.PickedSeedIds = vtk.vtkIdList()
        self.PickedSeeds = vtk.vtkPolyData()
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.Script = None

    def UndoCallback(self, obj):
        self.InitializeSeeds()
        self.PickedSeeds.Modified()
        self.vmtkRenderer.RenderWindow.Render()

    def PickCallback(self, obj):
        picker = vtk.vtkCellPicker()
        picker.SetTolerance(1E-4 * self._Surface.GetLength())
        eventPosition = self.vmtkRenderer.RenderWindowInteractor.GetEventPosition()
        result = picker.Pick(float(eventPosition[0]),float(eventPosition[1]),0.0,self.vmtkRenderer.Renderer)
        if result == 0:
            return
        pickPosition = picker.GetPickPosition()
        pickedCellPointIds = self._Surface.GetCell(picker.GetCellId()).GetPointIds()
        minDistance = 1E10
        pickedSeedId = -1
        for i in range(pickedCellPointIds.GetNumberOfIds()):
            distance = vtk.vtkMath.Distance2BetweenPoints(pickPosition,self._Surface.GetPoint(pickedCellPointIds.GetId(i)))
            if distance < minDistance:
                minDistance = distance
                pickedSeedId = pickedCellPointIds.GetId(i)
        if pickedSeedId == -1:
            pickedSeedId = pickedCellPointIds.GetId(0)
        self.PickedSeedIds.InsertNextId(pickedSeedId)
        point = self._Surface.GetPoint(pickedSeedId)
        self.PickedSeeds.GetPoints().InsertNextPoint(point)
        self.PickedSeeds.Modified()
        self.vmtkRenderer.RenderWindow.Render()

    def InitializeSeeds(self):
        self.PickedSeedIds.Initialize()
        self.PickedSeeds.Initialize()
        seedPoints = vtk.vtkPoints()
        self.PickedSeeds.SetPoints(seedPoints)

    def Execute(self):

        if (self._Surface == None):
            self.PrintError('vmtkPickPointSeedSelector Error: Surface not set.')
            return

        self._SourceSeedIds.Initialize()
        self._TargetSeedIds.Initialize()

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self.Script)

        glyphs = vtk.vtkGlyph3D()
        glyphSource = vtk.vtkSphereSource()
        glyphs.SetInputData(self.PickedSeeds)
        glyphs.SetSourceConnection(glyphSource.GetOutputPort())
        glyphs.SetScaleModeToDataScalingOff()
        glyphs.SetScaleFactor(self._Surface.GetLength()*0.01)
        glyphMapper = vtk.vtkPolyDataMapper()
        glyphMapper.SetInputConnection(glyphs.GetOutputPort())
        self.SeedActor = vtk.vtkActor()
        self.SeedActor.SetMapper(glyphMapper)
        self.SeedActor.GetProperty().SetColor(1.0,0.0,0.0)
        self.SeedActor.PickableOff()
        self.vmtkRenderer.Renderer.AddActor(self.SeedActor)

        ##self.vmtkRenderer.RenderWindowInteractor.AddObserver("KeyPressEvent", self.KeyPressed)
        self.vmtkRenderer.AddKeyBinding('u','Undo.',self.UndoCallback)
        self.vmtkRenderer.AddKeyBinding('space','Add points.',self.PickCallback)
        surfaceMapper = vtk.vtkPolyDataMapper()
        surfaceMapper.SetInputData(self._Surface)
        surfaceMapper.ScalarVisibilityOff()
        surfaceActor = vtk.vtkActor()
        surfaceActor.SetMapper(surfaceMapper)
        surfaceActor.GetProperty().SetOpacity(1.0)

        self.vmtkRenderer.Renderer.AddActor(surfaceActor)

        self.InputInfo('Please position the mouse and press space to add source points, \'u\' to undo\n')

        any = 0
        while any == 0:
            self.InitializeSeeds()
            self.vmtkRenderer.Render()
            any = self.PickedSeedIds.GetNumberOfIds()
        self._SourceSeedIds.DeepCopy(self.PickedSeedIds)

        self.InputInfo('Please position the mouse and press space to add target points, \'u\' to undo\n')

        any = 0
        while any == 0:
            self.InitializeSeeds()
            self.vmtkRenderer.Render()
            any = self.PickedSeedIds.GetNumberOfIds()
        self._TargetSeedIds.DeepCopy(self.PickedSeedIds)

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()


class vmtkOpenProfilesSeedSelector(vmtkSeedSelector):

    def __init__(self):
        vmtkSeedSelector.__init__(self)
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.Script = None

    def SetSeedIds(self,seedIds):
        self._SeedIds = seedIds

    def GetSeedIds(self):
        return self._SeedIds

    def Execute(self):

        if (self._Surface == None):
            self.PrintError('vmtkOpenProfilesSeedSelector Error: Surface not set.')
            return

        if (self._SeedIds == None):
            self.PrintError('vmtkOpenProfilesSeedSelector Error: SeedIds not set.')
            return

        self._SourceSeedIds.Initialize()
        self._TargetSeedIds.Initialize()

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self.Script)

        seedPoints = vtk.vtkPoints()
        for i in range(self._SeedIds.GetNumberOfIds()):
            seedPoints.InsertNextPoint(self._Surface.GetPoint(self._SeedIds.GetId(i)))
        seedPolyData = vtk.vtkPolyData()
        seedPolyData.SetPoints(seedPoints)
        labelsMapper = vtk.vtkLabeledDataMapper();
        labelsMapper.SetInputData(seedPolyData)
        labelsMapper.SetLabelModeToLabelIds()
        labelsActor = vtk.vtkActor2D()
        labelsActor.SetMapper(labelsMapper)

        self.vmtkRenderer.Renderer.AddActor(labelsActor)

        surfaceMapper = vtk.vtkPolyDataMapper()
        surfaceMapper.SetInputData(self._Surface)
        surfaceMapper.ScalarVisibilityOff()
        surfaceActor = vtk.vtkActor()
        surfaceActor.SetMapper(surfaceMapper)
        surfaceActor.GetProperty().SetOpacity(0.25)

        self.vmtkRenderer.Renderer.AddActor(surfaceActor)

        self.vmtkRenderer.Render()

        seedIdString = self.InputText("Please input list of inlet profile ids: ")
        separator = ' '
        if seedIdString.find(',') != -1:
            separator = ','
        seedIdList = seedIdString.split(separator)
        for seedIdString in seedIdList:
            self._SourceSeedIds.InsertNextId(int(seedIdString.strip()))

        seedIdString = self.InputText("Please input list of outlet profile ids (leave empty for all available profiles): ")
        if seedIdString.strip() == '':
            self.InputInfo("Selected outlets: ")
            for i in range(seedPoints.GetNumberOfPoints()):
                if self._SourceSeedIds.IsId(i) == -1:
                    self._TargetSeedIds.InsertNextId(i)
                    self.InputInfo("%d " % i)
            self.InputInfo("\n")
        else:
            seedIdList = seedIdString.split(separator)
            for seedIdString in seedIdList:
                self._TargetSeedIds.InsertNextId(int(seedIdString.strip()))

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()


class vmtkCarotidProfilesSeedSelector(vmtkSeedSelector):

    def SetSeedIds(self,seedIds):
        self._SeedIds = seedIds

    def GetSeedIds(self):
        return self._SeedIds

    def Execute(self):

        if (self._Surface == None):
            self.PrintError('vmtkCarotidProfilesSeedSelector Error: Surface not set.')
            return

        if (self._SeedIds == None):
            self.PrintError('vmtkCarotidProfilesSeedSelector Error: SeedIds not set.')
            return

        self._SourceSeedIds.Initialize()
        self._TargetSeedIds.Initialize()

        minZId = -1
        maxZId = -1
        minZ = 1E8
        maxZ = -1E8

        for id in range(self._SeedIds.GetNumberOfIds()):
            currentZ = self._Surface.GetPoint(self._SeedIds.GetId(id))[2]
            if (currentZ < minZ):
                minZ = currentZ
                minZId = id

            if (currentZ > maxZ):
                maxZ = currentZ
                maxZId = id

        self._SourceSeedIds.InsertNextId(minZId)
        self._TargetSeedIds.InsertNextId(maxZId)

        midZId = 3-(minZId+maxZId)
        self._TargetSeedIds.InsertNextId(midZId)


class vmtkNonManifoldSurfaceChecker(object):

    def __init__(self):

        self.Surface = 0

        self.NumberOfNonManifoldEdges = 0
        self.Report = 0
        self.NonManifoldEdgePointIds = vtk.vtkIdList()

        self.PrintError = None

    def Execute(self):

        if (self.Surface == 0):
            self.PrintError('NonManifoldSurfaceChecker error: Surface not set')
            return

        self.NonManifoldEdgesFound = 0
        self.Report = ''
        self.NonManifoldEdgePointIds.Initialize()

        neighborhoods = vtkvmtk.vtkvmtkNeighborhoods()
        neighborhoods.SetNeighborhoodTypeToPolyDataManifoldNeighborhood()
        neighborhoods.SetDataSet(self.Surface)
        neighborhoods.Build()

        neighborCellIds = vtk.vtkIdList()
        cellPointIds = vtk.vtkIdList()

        self.Surface.BuildCells()
        self.Surface.BuildLinks(0)

        numberOfNonManifoldEdges = 0

        for i in range(neighborhoods.GetNumberOfNeighborhoods()):

            neighborhood = neighborhoods.GetNeighborhood(i)

            for j in range(neighborhood.GetNumberOfPoints()):

                neighborId = neighborhood.GetPointId(j)

                if (i<neighborId):

                    neighborCellIds.Initialize()
                    self.Surface.GetCellEdgeNeighbors(-1,i,neighborId,neighborCellIds)

                    if (neighborCellIds.GetNumberOfIds()>2):

                        numberOfNonManifoldEdges = numberOfNonManifoldEdges + 1

                        self.Report = self.Report +  "Non-manifold edge found" + str(i) + ' ' + str(neighborId) + '.\n'

                        self.NonManifoldEdgePointIds.InsertNextId(i)
                        self.NonManifoldEdgePointIds.InsertNextId(neighborId)


class vmtkCenterlines(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Centerlines = None
        self.SeedSelector = None
        self.SeedSelectorName = 'pickpoint'
        self.FlipNormals = 0
        self.CapDisplacement = 0.0
        self.RadiusArrayName = 'MaximumInscribedSphereRadius'
        self.CostFunction = '1/R'
        self.AppendEndPoints = 0
        self.CheckNonManifold = 0

        self.Resampling = 0
        self.ResamplingStepLength = 1.0
        self.SimplifyVoronoi = 0

        self.EikonalSolutionArrayName = 'EikonalSolution'
        self.EdgeArrayName = 'EdgeArray'
        self.EdgePCoordArrayName = 'EdgePCoordArray'
        self.CostFunctionArrayName = 'CostFunctionArray'

        self.UseTetGen = 0
        self.TetGenDetectInter = 1

        self.DelaunayTessellation = None
        self.VoronoiDiagram = None
        self.PoleIds = None

        self.DelaunayTolerance = 0.001

        self.SourceIds = []
        self.TargetIds = []
        self.SourcePoints = []
        self.TargetPoints = []

        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.StopFastMarchingOnReachingTarget = 0

        self.SetScriptName('vmtkcenterlines')
        self.SetScriptDoc('compute centerlines from a branching tubular surface (see papers for details); seed points can be interactively selected on the surface, or specified as the barycenters of the open boundaries of the surface; if vmtk is compiled with support for TetGen, TetGen can be employed to compute the Delaunay tessellation of the input points')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['SeedSelectorName','seedselector','str',1,'["pickpoint","openprofiles","carotidprofiles","profileidlist","idlist","pointlist"]','seed point selection method (pickpoint: interactive; openprofiles: choose among barycenters of open profiles of the surface; carotidprofiles: open profiles are automatically selected based on their z-axis coordinate (lower to higher: CCA, ECA, ICA)); profileidlist: list of open profile ids (specified as argument to -sourceids and -targetids); idlist: list of surface point ids (specified as argument to -sourceids and -targetids); pointlist: list of surface points (specified as argument to -sourcepoints and -targetpoints)'],
            ['SourceIds','sourceids','int',-1,'','list of source point ids'],
            ['TargetIds','targetids','int',-1,'','list of target point ids'],
            ['SourcePoints','sourcepoints','float',-1,'','list of source point coordinates'],
            ['TargetPoints','targetpoints','float',-1,'','list of target point coordinates'],
            ['AppendEndPoints','endpoints','bool',1,'','toggle append open profile barycenters to centerlines'],
            ['CheckNonManifold','nonmanifoldcheck','bool',1,'','toggle checking the surface for non-manifold edges'],
            ['FlipNormals','flipnormals','bool',1,'','flip normals after outward normal computation; outward oriented normals must be computed for the removal of outer tetrahedra; the algorithm might fail so for weird geometries, so changing this might solve the problem'],
            ['CapDisplacement','capdisplacement','float',1,'','displacement of the center points of caps at open profiles along their normals (avoids the creation of degenerate tetrahedra)'],
            ['DelaunayTolerance','delaunaytolerance','float',1,'','tolerance for evaluating coincident points during Delaunay tessellation, evaluated as a fraction of the bounding box'],
            ['RadiusArrayName','radiusarray','str',1,'','name of the array where radius values of maximal inscribed spheres have to be stored'],
            ['AppendEndPoints','endpoints','bool',1,'','toggle append open profile barycenters to centerlines'],
            ['Resampling','resampling','bool',1,'','toggle centerlines resampling'],
            ['ResamplingStepLength','resamplingstep','float',1,'(0.0,)','distance between points in the resampled line'],
            ['DelaunayTessellation','delaunaytessellation','vtkUnstructuredGrid',1,'','optional input Delaunay tessellation'],
            ['SimplifyVoronoi','simplifyvoronoi','bool',1,'','toggle simplification of Voronoi diagram'],
            ['UseTetGen','usetetgen','bool',1,'','toggle use TetGen to compute Delaunay tessellation'],
            ['TetGenDetectInter','tetgendetectinter','bool',1,'','TetGen option'],
            ['CostFunction','costfunction','str',1,'','specify cost function to be minimized during centerline computation'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer'],
            ['PoleIds','poleids','vtkIdList',1],
            ['VoronoiDiagram','voronoidiagram','vtkPolyData',1,'','','vmtksurfacewriter'],
            ['StopFastMarchingOnReachingTarget','stopontarget','bool',1,'','terminate fast marching front propagation when the front reaches the target seed point. This can greatly speed up execution, but it only works when one target seed id is set.']])
        self.SetOutputMembers([
            ['Centerlines','o','vtkPolyData',1,'','the output centerlines','vmtksurfacewriter'],
            ['RadiusArrayName','radiusarray','str',1,'','name of the array where radius values of maximal inscribed spheres are stored'],
            ['EikonalSolutionArrayName','eikonalsolutionarray','str',1],
            ['EdgeArrayName','edgearray','str',1],
            ['EdgePCoordArrayName','edgepcoordarray','str',1],
            ['CostFunctionArrayName','costfunctionarray','str',1],
            ['DelaunayTessellation','delaunaytessellation','vtkUnstructuredGrid',1,'','','vmtkmeshwriter'],
            ['VoronoiDiagram','voronoidiagram','vtkPolyData',1,'','','vmtksurfacewriter'],
            ['PoleIds','poleids','vtkIdList',1]])

    def PrintProgress(self,obj,event):
        self.OutputProgress(obj.GetProgress(),10)

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if self.CheckNonManifold:
            self.PrintLog('NonManifold check.')
            nonManifoldChecker = vmtkNonManifoldSurfaceChecker()
            nonManifoldChecker.Surface = self.Surface
            nonManifoldChecker.PrintError = self.PrintError
            nonManifoldChecker.Execute()

            if (nonManifoldChecker.NumberOfNonManifoldEdges > 0):
                self.PrintLog(nonManifoldChecker.Report)
                return

        if not self.vmtkRenderer and self.SeedSelectorName in ['pickpoint','openprofiles']:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.PrintLog('Cleaning surface.')
        surfaceCleaner = vtk.vtkCleanPolyData()
        surfaceCleaner.SetInputData(self.Surface)
        surfaceCleaner.Update()

        self.PrintLog('Triangulating surface.')
        surfaceTriangulator = vtk.vtkTriangleFilter()
        surfaceTriangulator.SetInputConnection(surfaceCleaner.GetOutputPort())
        surfaceTriangulator.PassLinesOff()
        surfaceTriangulator.PassVertsOff()
        surfaceTriangulator.Update()

        centerlineInputSurface = surfaceTriangulator.GetOutput()

        capCenterIds = None

        if self.SeedSelectorName in ['openprofiles', 'carotidprofiles', 'pickpoint', 'profileidlist']:
            self.PrintLog('Capping surface.')
            surfaceCapper = vtkvmtk.vtkvmtkCapPolyData()
            surfaceCapper.SetInputConnection(surfaceTriangulator.GetOutputPort())
            surfaceCapper.SetDisplacement(self.CapDisplacement)
            surfaceCapper.SetInPlaneDisplacement(self.CapDisplacement)
            surfaceCapper.Update()
            centerlineInputSurface = surfaceCapper.GetOutput()
            capCenterIds = surfaceCapper.GetCapCenterIds()

        if self.SeedSelector:
            pass
        elif self.SeedSelectorName:
            if self.SeedSelectorName == 'pickpoint':
                self.SeedSelector = vmtkPickPointSeedSelector()
                self.SeedSelector.vmtkRenderer = self.vmtkRenderer
                self.SeedSelector.Script = self
            elif self.SeedSelectorName == 'openprofiles':
                self.SeedSelector = vmtkOpenProfilesSeedSelector()
                self.SeedSelector.vmtkRenderer = self.vmtkRenderer
                self.SeedSelector.Script = self
                self.SeedSelector.SetSeedIds(surfaceCapper.GetCapCenterIds())
            elif self.SeedSelectorName == 'carotidprofiles':
                self.SeedSelector = vmtkCarotidProfilesSeedSelector()
                self.SeedSelector.SetSeedIds(surfaceCapper.GetCapCenterIds())
            elif self.SeedSelectorName == 'idlist':
                self.SeedSelector = vmtkIdListSeedSelector()
                self.SeedSelector.SourceIds = self.SourceIds
                self.SeedSelector.TargetIds = self.TargetIds
            elif self.SeedSelectorName == 'pointlist':
                self.SeedSelector = vmtkPointListSeedSelector()
                self.SeedSelector.SourcePoints = self.SourcePoints
                self.SeedSelector.TargetPoints = self.TargetPoints
            elif self.SeedSelectorName != 'profileidlist':
                self.PrintError("SeedSelectorName unknown (available: pickpoint, openprofiles, carotidprofiles, profileidlist, idlist, pointlist)")
                return
        else:
            self.PrintError('vmtkCenterlines error: either SeedSelector or SeedSelectorName must be specified')
            return

        if self.SeedSelector:
            self.SeedSelector.SetSurface(centerlineInputSurface)
            self.SeedSelector.InputInfo = self.InputInfo
            self.SeedSelector.InputText = self.InputText
            self.SeedSelector.OutputText = self.OutputText
            self.SeedSelector.PrintError = self.PrintError
            self.SeedSelector.PrintLog = self.PrintLog
            self.SeedSelector.Execute()

            inletSeedIds = self.SeedSelector.GetSourceSeedIds()
            outletSeedIds = self.SeedSelector.GetTargetSeedIds()
        else:
            inletSeedIds = vtk.vtkIdList()
            outletSeedIds = vtk.vtkIdList()
            for id in self.SourceIds:
                inletSeedIds.InsertNextId(id)
            if self.TargetIds:
                for id in self.TargetIds:
                    outletSeedIds.InsertNextId(id)
            else:
                for i in range(capCenterIds.GetNumberOfIds()):
                    if i not in self.SourceIds:
                        outletSeedIds.InsertNextId(i)

        self.PrintLog('Computing centerlines.')
        self.InputInfo('Computing centerlines...')

        centerlineFilter = vtkvmtk.vtkvmtkPolyDataCenterlines()
        centerlineFilter.SetInputData(centerlineInputSurface)
        if self.SeedSelectorName in ['openprofiles','carotidprofiles','profileidlist']:
            centerlineFilter.SetCapCenterIds(capCenterIds)
        centerlineFilter.SetSourceSeedIds(inletSeedIds)
        centerlineFilter.SetTargetSeedIds(outletSeedIds)
        centerlineFilter.SetRadiusArrayName(self.RadiusArrayName)
        centerlineFilter.SetCostFunction(self.CostFunction)
        centerlineFilter.SetFlipNormals(self.FlipNormals)
        centerlineFilter.SetAppendEndPointsToCenterlines(self.AppendEndPoints)
        centerlineFilter.SetSimplifyVoronoi(self.SimplifyVoronoi)
        if self.StopFastMarchingOnReachingTarget == True:
            if outletSeedIds.GetNumberOfIds() != 1:
                self.PrintError('Parameter Conflict: cannot enable "StopFastMarchingOnReachingTarget" when there is more then one target seed set.')
            else:
                centerlineFilter.SetStopFastMarchingOnReachingTarget(self.StopFastMarchingOnReachingTarget)
        if self.DelaunayTessellation is not None:
            centerlineFilter.GenerateDelaunayTessellationOff()
            centerlineFilter.SetDelaunayTessellation(self.DelaunayTessellation)
            centerlineFilter.SetDelaunayTolerance(self.DelaunayTolerance)
        if (self.VoronoiDiagram is not None) and (self.PoleIds is not None):
            centerlineFilter.GenerateVoronoiDiagramOff()
            centerlineFilter.SetVoronoiDiagram(self.VoronoiDiagram)
            centerlineFilter.SetPoleIds(self.PoleIds)
            if self.SimplifyVoronoi == True:
                centerlineFilter.SetSimplifyVoronoi(0)
                self.PrintLog('Note: requested behavior (SimplifyVoronoi = True) over-ridden.',1)
                self.PrintLog('Cannot simplify Voronoi Diagram when precomputed input is specified.',1)
        if self.UseTetGen==1:
            self.PrintLog('Running TetGen.')
            from vmtk import vmtkscripts
            surfaceToMesh = vmtkscripts.vmtkSurfaceToMesh()
            surfaceToMesh.Surface = centerlineInputSurface
            surfaceToMesh.Execute()
            tetgen = vmtkscripts.vmtkTetGen()
            tetgen.Mesh = surfaceToMesh.Mesh
            tetgen.PLC = 1
            tetgen.NoMerge = 1
            tetgen.Quality = 0
            if self.TetGenDetectInter == 1:
                tetgen.DetectInter = 1
                tetgen.NoMerge = 0
            tetgen.OutputSurfaceElements = 0
            tetgen.Execute()
            centerlineFilter.GenerateDelaunayTessellationOff()
            centerlineFilter.SetDelaunayTessellation(tetgen.Mesh)
        centerlineFilter.SetCenterlineResampling(self.Resampling)
        centerlineFilter.SetResamplingStepLength(self.ResamplingStepLength)
        centerlineFilter.Update()

        self.Centerlines = centerlineFilter.GetOutput()
        self.VoronoiDiagram = centerlineFilter.GetVoronoiDiagram()
        self.DelaunayTessellation = centerlineFilter.GetDelaunayTessellation()
        self.PoleIds = centerlineFilter.GetPoleIds()

        self.EikonalSolutionArrayName = centerlineFilter.GetEikonalSolutionArrayName()
        self.EdgeArrayName = centerlineFilter.GetEdgeArrayName()
        self.EdgePCoordArrayName = centerlineFilter.GetEdgePCoordArrayName()
        self.CostFunctionArrayName = centerlineFilter.GetCostFunctionArrayName()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkimageinitialization as imageinitialization

#TODO: How do we test interaction?


@pytest.fixture()
def fast_marching_source_points():
    # this data was pulled from the aorta_image data set manually.
    return [71, 213, 16]


@pytest.fixture()
def fast_marching_target_points():
    # this data was pulled from the aorta_image data set manually.
    return [58, 78, 22,
            96, 78, 21]


@pytest.fixture()
def colliding_fronts_source_points():
    # this data was pulled from the aorta_image data set manually.
    return [71, 213, 16]


@pytest.fixture()
def colliding_fronts_target_points():
    # this data was pulled from the aorta_image data set manually.
    return [58, 78, 22]


def test_threshold_initialization_output_level_sets(aorta_image, compare_images):
    name = __name__ + '_test_threshold_initialization_output_level_sets.mha'
    initializer = imageinitialization.vmtkImageInitialization()
    initializer.Image = aorta_image
    initializer.Interactive = 0
    initializer.Method = 'threshold'
    initializer.LowerThreshold = 100
    initializer.UpperThreshold = 1000
    initializer.Execute()

    assert compare_images(initializer.InitialLevelSets, name) == True


def test_threshold_initialization_isosurface_value_is_zero(aorta_image):
    initializer = imageinitialization.vmtkImageInitialization()
    initializer.Image = aorta_image
    initializer.Interactive = 0
    initializer.Method = 'threshold'
    initializer.LowerThreshold = 100
    initializer.UpperThreshold = 1000
    initializer.Execute()

    assert initializer.IsoSurfaceValue == 0


def test_threshold_initialization_no_output_surface(aorta_image):
    initializer = imageinitialization.vmtkImageInitialization()
    initializer.Image = aorta_image
    initializer.Interactive = 0
    initializer.Method = 'threshold'
    initializer.LowerThreshold = 100
    initializer.UpperThreshold = 1000
    initializer.Execute()

    assert initializer.Surface is None


def test_isosurface_initialization_output_level_sets(aorta_image, compare_images):
    name = __name__ + '_test_isosurface_initialization_output_level_sets.mha'
    initializer = imageinitialization.vmtkImageInitialization()
    initializer.Image = aorta_image
    initializer.Interactive = 0
    initializer.Method = 'isosurface'
    initializer.IsoSurfaceValue = 500
    initializer.Execute()

    assert compare_images(initializer.InitialLevelSets, name) == True


def test_isosurface_initialization_output_isosurface_value(aorta_image):
    initializer = imageinitialization.vmtkImageInitialization()
    initializer.Image = aorta_image
    initializer.Interactive = 0
    initializer.Method = 'isosurface'
    initializer.IsoSurfaceValue = 500

    assert initializer.IsoSurfaceValue == 500

    initializer.Execute()

    # after execution the output isosurface value is reset to zero
    assert initializer.IsoSurfaceValue == 0


def test_fastmarching_initialization_output_level_sets(aorta_image, compare_images,
                                                       fast_marching_source_points,
                                                       fast_marching_target_points):
    name = __name__ + '_test_fastmarching_initialization_output_level_sets.mha'
    initializer = imageinitialization.vmtkImageInitialization()
    initializer.Image = aorta_image
    initializer.Interactive = 0
    initializer.Method = 'fastmarching'
    initializer.LowerThreshold = 700
    initializer.SourcePoints = fast_marching_source_points
    initializer.TargetPoints = fast_marching_target_points
    initializer.Execute()

    assert compare_images(initializer.InitialLevelSets, name) == True


def test_collidingfronts_initialization_output_level_sets(aorta_image, compare_images,
                                                          colliding_fronts_source_points,
                                                          colliding_fronts_target_points):
    name = __name__ + '_test_collidingfronts_initialization_output_level_sets.mha'
    initializer = imageinitialization.vmtkImageInitialization()
    initializer.Image = aorta_image
    initializer.Interactive = 0
    initializer.Method = 'collidingfronts'
    initializer.LowerThreshold = 700
    initializer.SourcePoints = colliding_fronts_source_points
    initializer.TargetPoints = colliding_fronts_target_points
    initializer.Execute()

    assert compare_images(initializer.InitialLevelSets, name) == True



## Program: VMTK
## Language:  Python

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Analytic tests for vtkvmtkStaticTemporalStreamTracer: particles are traced
## through synthetic velocity fields whose trajectories are known in closed
## form, so no reference data files are needed.

import numpy as np
import pytest
import vtk
from vmtk import vtkvmtk


def make_velocity_grid(velocity_per_timestep):
    '''An 11x11x11 unit-spacing grid centered on the origin, with one uniform
    vector array "Velocity_<i>" per requested time step.'''
    grid = vtk.vtkImageData()
    grid.SetDimensions(11, 11, 11)
    grid.SetSpacing(1.0, 1.0, 1.0)
    grid.SetOrigin(-5.0, -5.0, -5.0)
    numberOfPoints = grid.GetNumberOfPoints()
    for index, velocity in enumerate(velocity_per_timestep):
        array = vtk.vtkDoubleArray()
        array.SetName('Velocity_%d' % index)
        array.SetNumberOfComponents(3)
        array.SetNumberOfTuples(numberOfPoints)
        for i in range(numberOfPoints):
            array.SetTuple3(i, *velocity)
        grid.GetPointData().AddArray(array)
    return grid


def make_time_steps_table(times):
    table = vtk.vtkTable()
    indexColumn = vtk.vtkIntArray()
    indexColumn.SetName('index')
    timeColumn = vtk.vtkDoubleArray()
    timeColumn.SetName('time')
    for i, t in enumerate(times):
        indexColumn.InsertNextValue(i)
        timeColumn.InsertNextValue(t)
    table.AddColumn(indexColumn)
    table.AddColumn(timeColumn)
    return table


def make_seed(point):
    seedPoints = vtk.vtkPoints()
    seedPoints.InsertNextPoint(point)
    seed = vtk.vtkPolyData()
    seed.SetPoints(seedPoints)
    return seed


def run_tracer(grid, table, seed, maximum_propagation):
    tracer = vtkvmtk.vtkvmtkStaticTemporalStreamTracer()
    tracer.SetInputData(grid)
    tracer.SetSourceData(seed)
    tracer.SetTimeStepsTable(table)
    tracer.SetVectorPrefix('Velocity_')
    tracer.SetSeedTime(0.0)
    tracer.SetIntegrationDirectionToForward()
    tracer.SetIntegratorTypeToRungeKutta4()
    tracer.SetMaximumPropagation(maximum_propagation)
    tracer.SetMaximumNumberOfSteps(10000)
    tracer.SetIntegrationStepUnit(1)  # LENGTH_UNIT: steps in arc length, not cell lengths
    tracer.SetInitialIntegrationStep(0.05)
    tracer.Update()
    return tracer.GetOutput()


def test_uniform_steady_flow_traces_straight_line():
    grid = make_velocity_grid([(1.0, 0.0, 0.0), (1.0, 0.0, 0.0)])
    table = make_time_steps_table([0.0, 1.0])
    streamline = run_tracer(grid, table, make_seed((-4.0, 0.0, 0.0)), 8.0)

    assert streamline.GetNumberOfCells() == 1
    points = np.array([streamline.GetPoint(i) for i in range(streamline.GetNumberOfPoints())])
    assert points.shape[0] > 10
    # straight line along +x: y and z never deviate
    assert np.abs(points[:, 1]).max() < 1e-10
    assert np.abs(points[:, 2]).max() < 1e-10
    # the full propagation length is covered
    assert points[-1, 0] == pytest.approx(4.0, abs=1e-6)
    # integration time equals distance for unit speed
    timeArray = streamline.GetPointData().GetArray('IntegrationTime')
    assert timeArray is not None
    assert timeArray.GetValue(streamline.GetNumberOfPoints() - 1) == pytest.approx(8.0, abs=1e-6)


def test_time_varying_flow_blends_velocity_between_timesteps():
    # The velocity field turns from +x at t=0 to +y at t=1, so at any time in
    # between the temporally interpolated field is ((1-t), t, 0) everywhere.
    # The propagation length keeps the trace inside the [0, 1] time interval
    # (the full turn has an arc length of ~0.81).
    grid = make_velocity_grid([(1.0, 0.0, 0.0), (0.0, 1.0, 0.0)])
    table = make_time_steps_table([0.0, 1.0])
    streamline = run_tracer(grid, table, make_seed((0.0, 0.0, 0.0)), 0.6)

    points = np.array([streamline.GetPoint(i) for i in range(streamline.GetNumberOfPoints())])
    timeArray = streamline.GetPointData().GetArray('IntegrationTime')
    times = np.array([timeArray.GetValue(i) for i in range(streamline.GetNumberOfPoints())])
    velocityArray = streamline.GetPointData().GetArray('Velocity')
    velocities = np.array([velocityArray.GetTuple3(i) for i in range(streamline.GetNumberOfPoints())])

    # The recorded velocity must be the exact temporal blend of the two
    # timestep fields at the recorded integration time (the field is
    # spatially uniform, so spatial interpolation cannot mask errors here).
    expected_velocities = np.column_stack([1.0 - times, times, np.zeros_like(times)])
    assert np.allclose(velocities, expected_velocities, atol=1e-9)

    # Trajectory sanity: the path curves monotonically from +x toward +y in
    # the z=0 plane, staying close to the closed-form trajectory
    # x(t) = t - t^2/2, y(t) = t^2/2. The comparison is loose because the
    # tracer reconstructs integration time from arc length, which is only
    # approximate when the speed varies along the path.
    assert np.abs(points[:, 2]).max() < 1e-10
    steps = np.diff(points, axis=0)
    assert (steps[:, 0] > 0.0).all()
    assert (steps[1:, 1] > 0.0).all()
    assert np.allclose(points[:, 0], times - times ** 2 / 2.0, atol=0.05)
    assert np.allclose(points[:, 1], times ** 2 / 2.0, atol=0.05)



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagetonumpy.py,v $
## Language:  Python
## Date:      June 10, 2017
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo
##       The Jacobs Institute

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vtk.numpy_interface import dataset_adapter as dsa
from vtk.util import numpy_support
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes

try:
    import numpy as np
except ImportError:
    raise ImportError('Unable to Import vmtkimagetonumpy module, numpy is not installed')


# convenience class for nested dictionaries
class vividict(dict):
    def __missing__(self, key):
        value = self[key] = type(self)()
        return value


class vmtkImageToNumpy(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None
        self.ArrayDict = vividict()

        self.SetScriptName('vmtkImageToNumpy')
        self.SetScriptDoc('Takes a vtkImageData object and returns a nested dictionary of numpy arrays')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','the input image','vmtkimagereader']])
        self.SetOutputMembers([
            ['ArrayDict','o','dict',1,'','the output dictionary','vmtknumpywriter']])

    def Execute(self):

        if self.Image == None:
            self.PrintError('Error: no input image.')

        self.PrintLog('wrapping vtkDataObject')
        imWrapped = dsa.WrapDataObject(self.Image)

        self.PrintLog('setting origin')
        self.PrintLog('setting dimensions')
        self.PrintLog('setting spacing')

        self.ArrayDict['Origin'] = np.array(self.Image.GetOrigin())
        self.ArrayDict['Dimensions'] = np.array(self.Image.GetDimensions())
        self.ArrayDict['Spacing'] = np.array(self.Image.GetSpacing())

        self.PrintLog('writing point data: ')
        for pointKey in imWrapped.PointData.keys():

            flatPointArray = np.array(imWrapped.PointData.GetArray(pointKey))
            self.ArrayDict['PointData'][pointKey] = flatPointArray.reshape(self.ArrayDict['Dimensions'], order='F')


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkimageotsuthresholds as otsu
from vmtk.vtkvmtk import vtkvmtkITKVersion


@pytest.mark.skipif(vtkvmtkITKVersion.GetITKMajorVersion() > 4, reason="requires itk 4 or lower")
@pytest.mark.parametrize("thresholds,paramid", [
    (1, '0'),
    (2, '1'),
    (3, '2'),
])
def test_otsu_image_multiple_thresholds_itk4(aorta_image, compare_images, thresholds, paramid):
    name = __name__ + '_test_otsu_image_multiple_thresholds_' + paramid + '.mha'
    otsuer = otsu.vmtkImageOtsuThresholds()
    otsuer.Image = aorta_image
    otsuer.NumberOfThresholds = thresholds
    otsuer.Execute()

    assert compare_images(otsuer.Image, name) == True


@pytest.mark.skipif(vtkvmtkITKVersion.GetITKMajorVersion() > 4, reason="requires itk 4 or lower")
def test_otsu_image_256_histogram_bins_itk4(aorta_image, compare_images):
    name = __name__ + '_test_otsu_image_256_histogram_bins.mha'
    otsuer = otsu.vmtkImageOtsuThresholds()
    otsuer.Image = aorta_image
    otsuer.NumberOfHistogramBins = 256
    otsuer.Execute()

    assert compare_images(otsuer.Image, name) == True


@pytest.mark.skipif(vtkvmtkITKVersion.GetITKMajorVersion() < 5, reason="requires itk 5 or higher")
@pytest.mark.parametrize("thresholds,paramid", [
    (1, '0'),
    (2, '1'),
    (3, '2'),
])
def test_otsu_image_multiple_thresholds_itk5(aorta_image, compare_images, thresholds, paramid):
    name = __name__ + '_test_otsu_image_multiple_thresholds_' + paramid + '_itk5.mha'
    otsuer = otsu.vmtkImageOtsuThresholds()
    otsuer.Image = aorta_image
    otsuer.NumberOfThresholds = thresholds
    otsuer.Execute()

    assert compare_images(otsuer.Image, name) == True


@pytest.mark.skipif(vtkvmtkITKVersion.GetITKMajorVersion() < 5, reason="requires itk 5 or higher")
def test_otsu_image_256_histogram_bins_itk5(aorta_image, compare_images):
    name = __name__ + '_test_otsu_image_256_histogram_bins_itk5.mha'
    otsuer = otsu.vmtkImageOtsuThresholds()
    otsuer.Image = aorta_image
    otsuer.NumberOfHistogramBins = 256
    otsuer.Execute()

    assert compare_images(otsuer.Image, name) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkbranchgeometry.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:48:31 $
## Version:   $Revision: 1.7 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkBranchGeometry(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Centerlines = None
        self.GeometryData = None

        self.RadiusArrayName = 'MaximumInscribedSphereRadius'
        self.GroupIdsArrayName = 'GroupIds'
        self.BlankingArrayName = 'Blanking'

        self.LengthArrayName = 'Length'
        self.CurvatureArrayName = 'Curvature'
        self.TorsionArrayName = 'Torsion'
        self.TortuosityArrayName = 'Tortuosity'

        self.LineSmoothing = 0
        self.NumberOfSmoothingIterations = 100
        self.SmoothingFactor = 0.1

        self.SetScriptName('vmtkbranchgeometry')
        self.SetScriptDoc('compute geometric parameters for each branch of a tree. The script takes in input the centerlines already split into branches.')
        self.SetInputMembers([
            ['Centerlines','i','vtkPolyData',1,'','the input split centerlines','vmtksurfacereader'],
            ['RadiusArrayName','radiusarray','str',1,'','name of the array where centerline radius values are stored'],
            ['GroupIdsArrayName','groupidsarray','str',1,'','name of the array where centerline group ids are stored'],
            ['BlankingArrayName','blankingarray','str',1,'','name of the array where blanking information about branches is stored'],
            ['LengthArrayName','lengtharray','str',1,'','name of the array where the average length of each branch has to be stored'],
            ['CurvatureArrayName','curvaturearray','str',1,'','name of the array where the average curvature of each branch has to be stored'],
            ['TorsionArrayName','torsionarray','str',1,'','name of the array where the average torsion of each branch has to be stored'],
            ['TortuosityArrayName','tortuosityarray','str',1,'','name of the array where the average tortuosity of each branch, defined as the length of a line divided by the distance of its endpoints, has to be stored'],
            ['LineSmoothing','smoothing','bool',1,''],
            ['NumberOfSmoothingIterations','iterations','int',1,'(0,)'],
            ['SmoothingFactor','factor','float',1,'(0.0,)']
            ])
        self.SetOutputMembers([
            ['GeometryData','o','vtkPolyData',1,'','the output data set','vmtksurfacewriter'],
            ['LengthArrayName','lengtharray','str',1,'','name of the array where the average length of each branch is stored'],
            ['CurvatureArrayName','curvaturearray','str',1,'','name of the array where the average curvature of each branch is stored'],
            ['TorsionArrayName','torsionarray','str',1,'','name of the array where the average torsion of each branch is stored'],
            ['TortuosityArrayName','tortuosityarray','str',1,'','name of the array where the average tortuosity of each branch, defined as the length of a line divided by the distance of its endpoints minus one (L/D - 1), is stored']
            ])

    def Execute(self):

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        centerlineBranchGeometry = vtkvmtk.vtkvmtkCenterlineBranchGeometry()
        centerlineBranchGeometry.SetInputData(self.Centerlines)
        centerlineBranchGeometry.SetRadiusArrayName(self.RadiusArrayName)
        centerlineBranchGeometry.SetGroupIdsArrayName(self.GroupIdsArrayName)
        centerlineBranchGeometry.SetBlankingArrayName(self.BlankingArrayName)
        centerlineBranchGeometry.SetLengthArrayName(self.LengthArrayName)
        centerlineBranchGeometry.SetCurvatureArrayName(self.CurvatureArrayName)
        centerlineBranchGeometry.SetTorsionArrayName(self.TorsionArrayName)
        centerlineBranchGeometry.SetTortuosityArrayName(self.TortuosityArrayName)
        centerlineBranchGeometry.SetLineSmoothing(self.LineSmoothing)
        centerlineBranchGeometry.SetNumberOfSmoothingIterations(self.NumberOfSmoothingIterations)
        centerlineBranchGeometry.SetSmoothingFactor(self.SmoothingFactor)

        centerlineBranchGeometry.Update()

        self.GeometryData = centerlineBranchGeometry.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkimagevoipainter as voipainter
import vmtk.vmtkimageviewer as imageviewer

#TODO: Figure out why this fails.


@pytest.mark.skip(reason='this currently does not work. getting error "BoxExtent exceeds input WholeExtent"')
def test_voi_painter(aorta_image, compare_images):
    name = __name__ + '_test_voi_painter.mha'
    painter = voipainter.vmtkImageVOIPainter()
    painter.Image = aorta_image
    painter.Interactive = 0
    painter.PaintValue = 1500.0
    painter.BoxBounds = [190.0, 259.0, 110.0, 283.0, 12.0, 37.0]
    painter.Execute()

    print(painter.BoxBounds)
    viewer = imageviewer.vmtkImageViewer()
    viewer.Image = painter.Image
    viewer.Execute()
    # assert compare_images(painter.Image, name) == True



""" This module loads all the classes from the contrib library into its
namespace.  This is a required module."""

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import os
import vtk
from vmtk import vtkvmtk

if os.name == 'posix':
    from .libvtkvmtkContribPython import *
else:
    from .vtkvmtkContribPython import *



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimageinitialization.py,v $
## Language:  Python
## Date:      $Date: 2006/05/31 10:51:21 $
## Version:   $Revision: 1.19 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import math
import string
import vtk
from random import random

from vmtk import vtkvmtk

from vmtk import pypes


class vmtkImageInitialization(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None

        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.InitialLevelSets = None
        self.Surface = None

        self.MergedInitialLevelSets = None

        self.Interactive = 1
        self.Method = "collidingfronts"

        self.UpperThreshold = None
        self.LowerThreshold = None

        self.SourcePoints = []
        self.TargetPoints = []

        self.NegateImage = 0
        self.NumSeedsSelected = 0

        self.IsoSurfaceValue = 0.0

        self.ImageSeeder = None
        self.SurfaceViewer = None

        self.SetScriptName('vmtkimageinitialization')
        self.SetScriptDoc('interactivly create an initial levelset which has not been evolved to image gradients')
        self.SetInputMembers([
            ['Image','i','vtkImageData',1,'','','vmtkimagereader'],
            ['NegateImage','negate','bool',1,'','negate image values before initializing'],
            ['Interactive','interactive','bool',1,'',''],
            ['Method','method','str',1,'["isosurface","threshold","collidingfronts","fastmarching","seeds","multiplecollidingfronts"]',''],
            ['SourcePoints','sourcepoints','int',-1,'','list of source point IJK coordinates'],
            ['TargetPoints','targetpoints','int',-1,'','list of target point IJK coordinates'],
            ['UpperThreshold','upperthreshold','float',1,'','the value of the upper threshold to use for threshold, collidingfronts and fastmarching'],
            ['LowerThreshold','lowerthreshold','float',1,'','the value of the upper threshold to use for threshold, collidingfronts and fastmarching'],
            ['IsoSurfaceValue','isosurface','float',1,'','the isosurface value to adopt as the level set surface'],
            ['vmtkRenderer','renderer','vmtkRenderer',1]
            ])
        self.SetOutputMembers([
            ['IsoSurfaceValue','isosurfacevalue','float',1],
            ['InitialLevelSets','olevelsets','vtkImageData',1,'','','vmtkimagewriter'],
            ['Surface','osurface','vtkPolyData',1,'','','vmtksurfacewriter']
            ])

    def _SeedPickCallback(self, caller, event):
        '''change seed rgb color when two seeds have been selected'''
        self.NumSeedsSelected = self.ImageSeeder.Seeds.GetNumberOfPoints()
        if self.NumSeedsSelected % 2 == 0:
            self.ImageSeeder.SeedRGBColor = [random(), random(), random()]
            self.ImageSeeder.BuildView()

    def SeedInput(self,queryString,numberOfSeeds):
        invalid = 1
        while invalid == 1:
            invalid = 0
            self.InputInfo(queryString+' (click on the image while pressing Ctrl).\n')
            self.ImageSeeder.InitializeSeeds()
            self.vmtkRenderer.Render()

            if numberOfSeeds > 0:
                if self.ImageSeeder.Seeds.GetNumberOfPoints() != numberOfSeeds:
                    self.InputInfo('Invalid selection. Please place exactly '+str(numberOfSeeds)+' seeds.\n')
                    invalid = 1
                    continue

        seeds = vtk.vtkPolyData()
        seeds.DeepCopy(self.ImageSeeder.Seeds)

        return seeds

    def ThresholdValidator(self,text):
        if text == 'n':
            return 1
        if text == 'i':
            self.vmtkRenderer.Render()
            return 0
        try:
            float(text)
        except ValueError:
            return 0
        return 1

    def ThresholdInput(self,queryString):

        thresholdString = self.InputText(queryString,self.ThresholdValidator)

        threshold = None
        if thresholdString != 'n':
            threshold = float(thresholdString)

        return threshold

    def IsosurfaceInitialize(self):

        self.PrintLog('Isosurface initialization.')

        if self.Interactive:
            queryString = "Please input isosurface level (\'n\' for none): "
            self.IsoSurfaceValue = self.ThresholdInput(queryString)

        imageMathematics = vtk.vtkImageMathematics()
        imageMathematics.SetInputData(self.Image)
        imageMathematics.SetConstantK(-1.0)
        imageMathematics.SetOperationToMultiplyByK()
        imageMathematics.Update()

        subtract = vtk.vtkImageMathematics()
        subtract.SetInputConnection(imageMathematics.GetOutputPort())
        subtract.SetOperationToAddConstant()
        subtract.SetConstantC(self.IsoSurfaceValue)
        subtract.Update()

        self.InitialLevelSets = vtk.vtkImageData()
        self.InitialLevelSets.DeepCopy(subtract.GetOutput())

        self.IsoSurfaceValue = 0.0

    def ThresholdInitialize(self):

        self.PrintLog('Threshold initialization.')

        if self.Interactive:
            queryString = "Please input lower threshold (\'n\' for none): "
            self.LowerThreshold = self.ThresholdInput(queryString)

            queryString = "Please input upper threshold (\'n\' for none): "
            self.UpperThreshold = self.ThresholdInput(queryString)

        scalarRange = self.Image.GetScalarRange()

        thresholdedImage = self.Image

        if self.LowerThreshold != None or self.UpperThreshold != None:
            threshold = vtk.vtkImageThreshold()
            threshold.SetInputData(self.Image)
            if self.LowerThreshold != None and self.UpperThreshold != None:
                threshold.ThresholdBetween(self.LowerThreshold,self.UpperThreshold)
            elif self.LowerThreshold != None:
                threshold.ThresholdByUpper(self.LowerThreshold)
            elif self.UpperThreshold != None:
                threshold.ThresholdByLower(self.UpperThreshold)
            threshold.ReplaceInOn()
            threshold.ReplaceOutOn()
            threshold.SetInValue(-1.0)
            threshold.SetOutValue(1.0)
            threshold.Update()

            thresholdedImage = threshold.GetOutput()

        self.InitialLevelSets = vtk.vtkImageData()
        self.InitialLevelSets.DeepCopy(thresholdedImage)

        self.IsoSurfaceValue = 0.0

    def FastMarchingInitialize(self):

        self.PrintLog('Fast marching initialization.')

        sourceSeedIds = vtk.vtkIdList()
        targetSeedIds = vtk.vtkIdList()
        if self.Interactive:
            queryString = "Please input lower threshold (\'n\' for none): "
            self.LowerThreshold = self.ThresholdInput(queryString)

            queryString = "Please input upper threshold (\'n\' for none): "
            self.UpperThreshold = self.ThresholdInput(queryString)

            queryString = 'Please place source seeds'
            sourceSeeds = self.SeedInput(queryString,0)

            queryString = 'Please place target seeds'
            targetSeeds = self.SeedInput(queryString,0)

            for i in range(sourceSeeds.GetNumberOfPoints()):
                sourceSeedIds.InsertNextId(self.Image.FindPoint(sourceSeeds.GetPoint(i)))

            for i in range(targetSeeds.GetNumberOfPoints()):
                targetSeedIds.InsertNextId(self.Image.FindPoint(targetSeeds.GetPoint(i)))

        else:
            for i in range(len(self.SourcePoints)//3):
                sourceSeedIds.InsertNextId(self.Image.ComputePointId([self.SourcePoints[3*i+0],self.SourcePoints[3*i+1],self.SourcePoints[3*i+2]]))
            for i in range(len(self.TargetPoints)//3):
                targetSeedIds.InsertNextId(self.Image.ComputePointId([self.TargetPoints[3*i+0],self.TargetPoints[3*i+1],self.TargetPoints[3*i+2]]))

        scalarRange = self.Image.GetScalarRange()

        thresholdedImage = self.Image

        if (self.LowerThreshold is not None) | (self.UpperThreshold is not None):
            threshold = vtk.vtkImageThreshold()
            threshold.SetInputData(self.Image)
            if (self.LowerThreshold is not None) & (self.UpperThreshold is not None):
                threshold.ThresholdBetween(self.LowerThreshold,self.UpperThreshold)
            elif (self.LowerThreshold is not None):
                threshold.ThresholdByUpper(self.LowerThreshold)
            elif (self.UpperThreshold is not None):
                threshold.ThresholdByLower(self.UpperThreshold)
            threshold.ReplaceInOff()
            threshold.ReplaceOutOn()
            threshold.SetOutValue(scalarRange[0] - scalarRange[1])
            threshold.Update()

            scalarRange = threshold.GetOutput().GetScalarRange()

            thresholdedImage = threshold.GetOutput()

        scale = 1.0
        if scalarRange[1]-scalarRange[0] > 0.0:
            scale = 1.0 / (scalarRange[1]-scalarRange[0])

        shiftScale = vtk.vtkImageShiftScale()
        shiftScale.SetInputData(thresholdedImage)
        shiftScale.SetShift(-scalarRange[0])
        shiftScale.SetScale(scale)
        shiftScale.SetOutputScalarTypeToFloat()
        shiftScale.Update()

        speedImage = shiftScale.GetOutput()

        fastMarching = vtkvmtk.vtkvmtkFastMarchingUpwindGradientImageFilter()
        fastMarching.SetInputData(speedImage)
        fastMarching.SetSeeds(sourceSeedIds)
        fastMarching.GenerateGradientImageOff()
        fastMarching.SetTargetOffset(100.0)
        fastMarching.SetTargets(targetSeedIds)
        if targetSeedIds.GetNumberOfIds() > 0:
            fastMarching.SetTargetReachedModeToOneTarget()
        else:
            fastMarching.SetTargetReachedModeToNoTargets()
        fastMarching.Update()

        subtract = vtk.vtkImageMathematics()
        subtract.SetInputConnection(fastMarching.GetOutputPort())
        subtract.SetOperationToAddConstant()
        subtract.SetConstantC(-fastMarching.GetTargetValue())
        subtract.Update()

        self.InitialLevelSets = vtk.vtkImageData()
        self.InitialLevelSets.DeepCopy(subtract.GetOutput())

        self.IsoSurfaceValue = 0.0

    def CollidingFrontsInitialize(self):

        self.PrintLog('Colliding fronts initialization.')

        seedIds1 = vtk.vtkIdList()
        seedIds2 = vtk.vtkIdList()

        if self.Interactive:
            queryString = "Please input lower threshold (\'n\' for none): "
            self.LowerThreshold = self.ThresholdInput(queryString)

            queryString = "Please input upper threshold (\'n\' for none): "
            self.UpperThreshold = self.ThresholdInput(queryString)

            queryString = 'Please place two seeds'
            seeds = self.SeedInput(queryString,2)

            seedIds1.InsertNextId(self.Image.FindPoint(seeds.GetPoint(0)))
            seedIds2.InsertNextId(self.Image.FindPoint(seeds.GetPoint(1)))

        else:
            seedIds1.InsertNextId(self.Image.ComputePointId([self.SourcePoints[0],self.SourcePoints[1],self.SourcePoints[2]]))
            seedIds2.InsertNextId(self.Image.ComputePointId([self.TargetPoints[0],self.TargetPoints[1],self.TargetPoints[2]]))

        scalarRange = self.Image.GetScalarRange()

        thresholdedImage = self.Image

        if (self.LowerThreshold is not None) | (self.UpperThreshold is not None):
            threshold = vtk.vtkImageThreshold()
            threshold.SetInputData(self.Image)
            if (self.LowerThreshold is not None) & (self.UpperThreshold is not None):
                threshold.ThresholdBetween(self.LowerThreshold,self.UpperThreshold)
            elif (self.LowerThreshold is not None):
                threshold.ThresholdByUpper(self.LowerThreshold)
            elif (self.UpperThreshold is not None):
                threshold.ThresholdByLower(self.UpperThreshold)
            threshold.ReplaceInOff()
            threshold.ReplaceOutOn()
            threshold.SetOutValue(scalarRange[0] - scalarRange[1])
            threshold.Update()

            scalarRange = threshold.GetOutput().GetScalarRange()

            thresholdedImage = threshold.GetOutput()

        scale = 1.0
        if scalarRange[1]-scalarRange[0] > 0.0:
            scale = 1.0 / (scalarRange[1]-scalarRange[0])

        shiftScale = vtk.vtkImageShiftScale()
        shiftScale.SetInputData(thresholdedImage)
        shiftScale.SetShift(-scalarRange[0])
        shiftScale.SetScale(scale)
        shiftScale.SetOutputScalarTypeToFloat()
        shiftScale.Update()

        speedImage = shiftScale.GetOutput()

        collidingFronts = vtkvmtk.vtkvmtkCollidingFrontsImageFilter()
        collidingFronts.SetInputData(speedImage)
        collidingFronts.SetSeeds1(seedIds1)
        collidingFronts.SetSeeds2(seedIds2)
        collidingFronts.ApplyConnectivityOn()
        collidingFronts.StopOnTargetsOn()
        collidingFronts.Update()

        subtract = vtk.vtkImageMathematics()
        subtract.SetInputConnection(collidingFronts.GetOutputPort())
        subtract.SetOperationToAddConstant()
        subtract.SetConstantC(-10.0 * collidingFronts.GetNegativeEpsilon())
        subtract.Update()

        self.InitialLevelSets = vtk.vtkImageData()
        self.InitialLevelSets.DeepCopy(subtract.GetOutput())

        self.IsoSurfaceValue = 0.0

    def MultipleCollidingFrontsInitialize(self):

        self.PrintLog('Multiple colliding fronts initialization.')
        if self.Interactive:
            queryString = "Please input lower threshold (\'n\' for none): "
            self.LowerThreshold = self.ThresholdInput(queryString)
            queryString = "Please input upper threshold (\'n\' for none): "
            self.UpperThreshold = self.ThresholdInput(queryString)

        scalarRange = self.Image.GetScalarRange()
        thresholdedImage = self.Image

        if (self.LowerThreshold is not None) | (self.UpperThreshold is not None):
            threshold = vtk.vtkImageThreshold()
            threshold.SetInputData(self.Image)
            if (self.LowerThreshold is not None) & (self.UpperThreshold is not None):
                threshold.ThresholdBetween(self.LowerThreshold,self.UpperThreshold)
            elif (self.LowerThreshold is not None):
                threshold.ThresholdByUpper(self.LowerThreshold)
            elif (self.UpperThreshold is not None):
                threshold.ThresholdByLower(self.UpperThreshold)
            threshold.ReplaceInOff()
            threshold.ReplaceOutOn()
            threshold.SetOutValue(scalarRange[0] - scalarRange[1])
            threshold.Update()

            scalarRange = threshold.GetOutput().GetScalarRange()
            thresholdedImage = threshold.GetOutput()

        scale = 1.0
        if scalarRange[1]-scalarRange[0] > 0.0:
            scale = 1.0 / (scalarRange[1]-scalarRange[0])

        shiftScale = vtk.vtkImageShiftScale()
        shiftScale.SetInputData(thresholdedImage)
        shiftScale.SetShift(-scalarRange[0])
        shiftScale.SetScale(scale)
        shiftScale.SetOutputScalarTypeToFloat()
        shiftScale.Update()
        speedImage = shiftScale.GetOutput()

        seedIdsList = []
        if self.Interactive:
            seeds = None
            while not seeds:
                self.ImageSeeder.Seeds.AddObserver('ModifiedEvent', self._SeedPickCallback)
                queryString = 'Please place an even number of seeds'
                seeds = self.SeedInput(queryString,0)
            for pointIds in range(self.NumSeedsSelected//2):
                seedIds1 = vtk.vtkIdList()
                seedIds2 = vtk.vtkIdList()
                seedIds1.InsertNextId(self.Image.FindPoint(seeds.GetPoint(pointIds*2)))
                seedIds2.InsertNextId(self.Image.FindPoint(seeds.GetPoint((pointIds*2)+1)))
                seedIdsList.append([seedIds1, seedIds2])
        else:
            self.PrintError('Non Interactive use not permitted when using Multiple Colliding '\
                            'Fronts Initialization method.')

        lsList = []
        for seedIds1, seedIds2 in seedIdsList:
            collidingFronts = vtkvmtk.vtkvmtkCollidingFrontsImageFilter()
            collidingFronts.SetInputData(speedImage)
            collidingFronts.SetSeeds1(seedIds1)
            collidingFronts.SetSeeds2(seedIds2)              
            collidingFronts.ApplyConnectivityOn()
            collidingFronts.StopOnTargetsOn()
            collidingFronts.Update()

            subtract = vtk.vtkImageMathematics()
            subtract.SetInputConnection(collidingFronts.GetOutputPort())
            subtract.SetOperationToAddConstant()
            subtract.SetConstantC(-10.0 * collidingFronts.GetNegativeEpsilon())
            subtract.Update()

            singleLevelSets = vtk.vtkImageData()
            singleLevelSets.DeepCopy(subtract.GetOutput())
            lsList.append(singleLevelSets)

        for counter, levelSet in enumerate(lsList):
            if counter == 0:
                combined = vtk.vtkImageData()
                combined.DeepCopy(levelSet)
                continue
            else:
                combine = vtk.vtkImageMathematics()
                combine.SetOperationToAdd()
                combine.SetInput1Data(combined)
                combine.SetInput2Data(levelSet)
                combine.Update()
                combined = vtk.vtkImageData()
                combined.DeepCopy(combine.GetOutput())
                

        self.InitialLevelSets = vtk.vtkImageData()
        self.InitialLevelSets.DeepCopy(combined)

        self.IsoSurfaceValue = 0.0 

    def SeedInitialize(self):

        self.PrintLog('Seed initialization.')

        seedIds = vtk.vtkIdList()
        if self.Interactive:
            queryString = 'Please place seeds'
            seeds = self.SeedInput(queryString,0)
            for i in range(seeds.GetNumberOfPoints()):
                seedIds.InsertNextId(self.Image.FindPoint(seeds.GetPoint(i)))
        else:
            for i in range(len(self.SourcePoints)//3):
                seedIds.InsertNextId(self.Image.ComputePointId([self.SourcePoints[3*i+0],self.SourcePoints[3*i+1],self.SourcePoints[3*i+2]]))
            for i in range(len(self.TargetPoints)//3):
                seedIds.InsertNextId(self.Image.ComputePointId([self.TargetPoints[3*i+0],self.TargetPoints[3*i+1],self.TargetPoints[3*i+2]]))

        self.InitialLevelSets = vtk.vtkImageData()
        self.InitialLevelSets.DeepCopy(self.Image)

        levelSetsInputScalars = self.InitialLevelSets.GetPointData().GetScalars()
        levelSetsInputScalars.FillComponent(0,1.0)

        dimensions = self.Image.GetDimensions()
        for i in range(seedIds.GetNumberOfIds()):
            levelSetsInputScalars.SetComponent(seedIds.GetId(i),0,-1.0)

        dilateErode = vtk.vtkImageDilateErode3D()
        dilateErode.SetInputData(self.InitialLevelSets)
        dilateErode.SetDilateValue(-1.0)
        dilateErode.SetErodeValue(1.0)
        dilateErode.SetKernelSize(3,3,3)
        dilateErode.Update()

        self.InitialLevelSets.DeepCopy(dilateErode.GetOutput())

        self.IsoSurfaceValue = 0.0

    def DisplayLevelSetSurface(self,levelSets):

        value = 0.0
        marchingCubes = vtk.vtkMarchingCubes()
        marchingCubes.SetInputData(levelSets)
        marchingCubes.SetValue(0,value)
        marchingCubes.Update()

        self.Surface = marchingCubes.GetOutput()

        self.OutputText('Displaying.\n')

        self.SurfaceViewer.Surface = marchingCubes.GetOutput()
        self.SurfaceViewer.Display = 0
        self.SurfaceViewer.Opacity = 0.5
        self.SurfaceViewer.BuildView()

    def InitializationTypeValidator(self,text):
        if text in ['0','1','2','3','4','5']:
            return 1
        return 0

    def YesNoValidator(self,text):
        if text in ['n','y']:
            return 1
        return 0

    def MergeLevelSets(self):

        if self.MergedInitialLevelSets == None:
            self.MergedInitialLevelSets = vtk.vtkImageData()
            self.MergedInitialLevelSets.DeepCopy(self.InitialLevelSets)
        else:
            minFilter = vtk.vtkImageMathematics()
            minFilter.SetOperationToMin()
            minFilter.SetInput1Data(self.MergedInitialLevelSets)
            minFilter.SetInput2Data(self.InitialLevelSets)
            minFilter.Update()
            self.MergedInitialLevelSets = minFilter.GetOutput()

    def Execute(self):
        from vmtk import vmtkscripts
        if self.Image == None:
            self.PrintError('Error: no Image.')

        cast = vtk.vtkImageCast()
        cast.SetInputData(self.Image)
        cast.SetOutputScalarTypeToFloat()
        cast.Update()
        self.Image = cast.GetOutput()

        if self.NegateImage:
            scalarRange = self.Image.GetScalarRange()
            negate = vtk.vtkImageMathematics()
            negate.SetInputData(self.Image)
            negate.SetOperationToMultiplyByK()
            negate.SetConstantK(-1.0)
            negate.Update()
            shiftScale = vtk.vtkImageShiftScale()
            shiftScale.SetInputConnection(negate.GetOutputPort())
            shiftScale.SetShift(scalarRange[1]+scalarRange[0])
            shiftScale.SetOutputScalarTypeToFloat()
            shiftScale.Update()
            self.Image = shiftScale.GetOutput()

        if self.Interactive:
            if not self.vmtkRenderer:
                self.vmtkRenderer = vmtkscripts.vmtkRenderer()
                self.vmtkRenderer.Initialize()
                self.OwnRenderer = 1

            self.vmtkRenderer.RegisterScript(self)

            if not self.ImageSeeder:
                self.ImageSeeder = vmtkscripts.vmtkImageSeeder()
                self.ImageSeeder.vmtkRenderer = self.vmtkRenderer
                self.ImageSeeder.Image = self.Image
                self.ImageSeeder.Display = 0
                self.ImageSeeder.Execute()
                ##self.ImageSeeder.Display = 1
                self.ImageSeeder.BuildView()

            if not self.SurfaceViewer:
                self.SurfaceViewer = vmtkscripts.vmtkSurfaceViewer()
                self.SurfaceViewer.vmtkRenderer = self.vmtkRenderer

            initializationMethods = {
                                '0': self.CollidingFrontsInitialize,
                                '1': self.MultipleCollidingFrontsInitialize,
                                '2': self.FastMarchingInitialize,
                                '3': self.ThresholdInitialize,
                                '4': self.IsosurfaceInitialize,
                                '5': self.SeedInitialize,
                                }

            endInitialization = False
            while not endInitialization:
                queryString = 'Please choose initialization type: \n'\
                              '0: colliding fronts;\n'\
                              '1: multiple colliding fronts;\n'\
                              '2: fast marching;\n'\
                              '3: threshold;\n'\
                              '4: isosurface;\n'\
                              '5: seed;\n'
                initializationType = self.InputText(queryString,self.InitializationTypeValidator)
                initializationMethods[initializationType]()
                self.DisplayLevelSetSurface(self.InitialLevelSets)
                queryString = 'Accept initialization? (y/n): '
                inputString = self.InputText(queryString,self.YesNoValidator)
                if inputString == 'y':
                    self.MergeLevelSets()
                    self.DisplayLevelSetSurface(self.MergedInitialLevelSets)
                queryString = 'Initialize another branch? (y/n): '
                inputString = self.InputText(queryString,self.YesNoValidator)
                if inputString == 'y':
                    endInitialization = False
                elif inputString == 'n':
                    endInitialization = True

            self.InitialLevelSets = self.MergedInitialLevelSets
            self.MergedInitialLevelSets = None

        else:
            if self.Method == "collidingfronts":
                self.CollidingFrontsInitialize()
            elif self.Method == "multiplecollidingfronts":
                self.MultipleCollidingFrontsInitialize()
            elif self.Method == "fastmarching":
                self.FastMarchingInitialize()
            elif self.Method == "threshold":
                self.ThresholdInitialize()
            elif self.Method == "isosurface":
                self.IsosurfaceInitialize()
            elif self.Method == "seeds":
                self.SeedInitialize()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagesmoothing.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkPythonScript(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None
        self.Image2 = None
        self.Surface = None
        self.Surface2 = None
        self.Mesh = None
        self.Mesh2 = None
        self.PythonScriptFileName = ''

        self.SetScriptName('vmtkpythonscript')
        self.SetScriptDoc('execute a python script contained in a file')
        self.SetInputMembers([
            ['Image','image','vtkImageData',1,'','the input image','vmtkimagereader'],
            ['Image2','image2','vtkImageData',1,'','the second input image','vmtkimagereader'],
            ['Surface','surface','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['Surface2','surface2','vtkPolyData',1,'','the second input surface','vmtksurfacereader'],
            ['Mesh','mesh','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['Mesh2','mesh2','vtkUnstructuredGrid',1,'','the second input mesh','vmtkmeshreader'],
            ['PythonScriptFileName','scriptfile','str',1,'','the name of the file were the Python script resides']
            ])
        self.SetOutputMembers([
            ['Image','oimage','vtkImageData',1,'','the output image','vmtkimagewriter'],
            ['Surface','osurface','vtkPolyData',1,'','the output surface','vmtksurfacewriter'],
            ['Mesh','omesh','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter']
            ])

    def Execute(self):

        if self.PythonScriptFileName == '':
            self.PrintError('Error: no PythonScriptFileName')

        try:
            exec(compile(open(self.PythonScriptFileName).read(), self.PythonScriptFileName, 'exec'))
        except Exception as error:
            self.PrintError("Python script error: %s" % error)


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfacecurvature.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys
from vmtk import vtkvmtk

from vmtk import pypes


class vmtkSurfaceCurvature(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.ReferenceSurface = None

        self.CurvatureType = 'mean'

        self.AbsoluteCurvature = 0
        self.CurvatureOnBoundaries = 0
        self.MedianFiltering = 0
        self.BoundedReciprocal = 0
        self.Epsilon = 1.0
        self.Offset = 0.0

        self.SetScriptName('vmtksurfacecurvature')
        self.SetScriptDoc('compute curvature of an input surface, optionally reporting results back to a reference surface (by point id)')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['ReferenceSurface','r','vtkPolyData',1,'','the reference surface','vmtksurfacereader'],
            ['CurvatureType','type','str',1,'["mean","gaussian","maximum","minimum"]','the type of surface curvature to compute'],
            ['AbsoluteCurvature','absolute','bool',1,'','output the absolute value of curvature'],
            ['MedianFiltering','median','bool',1,'','output curvature after median filtering to suppress numerical noise speckles'],
            ['CurvatureOnBoundaries','boundaries','bool',1,'','compute curvature on boundaries'],
            ['BoundedReciprocal','reciprocal','bool',1,'','output bounded reciprocal of curvature'],
            ['Epsilon','epsilon','float',1,'','bounded reciprocal epsilon at the denominator'],
            ['Offset','offset','float',1,'','offset curvature by the specified value']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        curvatureFilter = vtk.vtkCurvatures()
        curvatureFilter.SetInputData(self.Surface)
        if self.CurvatureType == 'mean':
            curvatureFilter.SetCurvatureTypeToMean()
        elif self.CurvatureType == 'gaussian':
            curvatureFilter.SetCurvatureTypeToGaussian()
        elif self.CurvatureType == 'maximum':
            curvatureFilter.SetCurvatureTypeToMaximum()
        elif self.CurvatureType == 'minimum':
            curvatureFilter.SetCurvatureTypeToMinimum()
        curvatureFilter.Update()

        activeScalars = curvatureFilter.GetOutput().GetPointData().GetScalars()
        activeScalars.SetName('Curvature')

        if self.AbsoluteCurvature:
            for i in range(activeScalars.GetNumberOfTuples()):
                value = activeScalars.GetTuple1(i)
                activeScalars.SetTuple1(i,abs(value))

        neighborhoods = None
        if not self.CurvatureOnBoundaries or self.MedianFiltering:
            neighborhoods = vtkvmtk.vtkvmtkNeighborhoods()
            neighborhoods.SetNeighborhoodTypeToPolyDataManifoldNeighborhood()
            neighborhoods.SetDataSet(self.Surface)
            neighborhoods.Build()

        if not self.CurvatureOnBoundaries:
            boundaryExtractor = vtkvmtk.vtkvmtkPolyDataBoundaryExtractor()
            boundaryExtractor.SetInputData(self.Surface)
            boundaryExtractor.Update()
            boundaryIdsArray = vtk.vtkIdTypeArray.SafeDownCast(boundaryExtractor.GetOutput().GetPointData().GetScalars())
            boundaryIds = vtk.vtkIdList()
            boundaryIds.SetNumberOfIds(boundaryIdsArray.GetNumberOfTuples())
            for i in range(boundaryIdsArray.GetNumberOfTuples()):
                boundaryIds.SetId(i,boundaryIdsArray.GetValue(i))
            self.Surface.BuildLinks()
            for i in range(boundaryIds.GetNumberOfIds()):
                pointId = boundaryIds.GetId(i)
                neighborhood = neighborhoods.GetNeighborhood(pointId)
                values = []
                for j in range(neighborhood.GetNumberOfPoints()):
                    neighborId = neighborhood.GetPointId(j)
                    if boundaryIds.IsId(neighborId) != -1:
                        continue
                    value = activeScalars.GetTuple1(neighborId)
                    values.append(value)
                values.sort()
                if not values:
                    continue
                medianValue = values[(len(values) - 1)//2]
                activeScalars.SetTuple1(pointId,medianValue)

        if self.MedianFiltering:
            for i in range(neighborhoods.GetNumberOfNeighborhoods()):
                neighborhood = neighborhoods.GetNeighborhood(i)
                values = []
                for j in range(neighborhood.GetNumberOfPoints()):
                    neighborId = neighborhood.GetPointId(j)
                    value = activeScalars.GetTuple1(neighborId)
                    values.append(value)
                values.sort()
                if not values:
                    continue
                medianValue = values[(len(values) - 1)//2]
                activeScalars.SetTuple1(i,medianValue)

        if self.BoundedReciprocal:
            for i in range(activeScalars.GetNumberOfTuples()):
                value = activeScalars.GetTuple1(i)
                reciprocalValue = 1.0 / (self.Epsilon + value)
                activeScalars.SetTuple1(i,reciprocalValue)

        if self.Offset:
            for i in range(activeScalars.GetNumberOfTuples()):
                value = activeScalars.GetTuple1(i)
                activeScalars.SetTuple1(i,value + self.Offset)

        if self.ReferenceSurface == None:
            self.Surface.GetPointData().AddArray(activeScalars)
        else:
            self.ReferenceSurface.GetPointData().AddArray(activeScalars)
            self.Surface = self.ReferenceSurface


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkimageshiftscale as imageshiftscale


@pytest.mark.parametrize("outputType,expected_type", [
    ("float", 'float'),
    ("double", 'double'),
    ("uchar", 'unsigned char'),
    ("char", 'char'),
    ("ushort", 'unsigned short'),
    ("short", 'short'),
    ("long", 'long'),
    ("ulong", 'unsigned long'),
    ("int", 'int'),
    ("uint", 'unsigned int'),
])
def test_cast_output_as_types(aorta_image, outputType, expected_type):
    shifter = imageshiftscale.vmtkImageShiftScale()
    shifter.Image = aorta_image
    shifter.OutputType = outputType
    shifter.Execute()

    assert shifter.Image.GetScalarTypeAsString() == expected_type


def test_shift_value_positive(aorta_image, compare_images):
    name = __name__ + '_test_shift_value_positive.mha'
    shifter = imageshiftscale.vmtkImageShiftScale()
    shifter.Image = aorta_image
    shifter.Shift = 10.3
    shifter.Execute()

    assert compare_images(shifter.Image, name) == True


def test_shift_value_negative(aorta_image, compare_images):
    name = __name__ + '_test_shift_value_negative.mha'
    shifter = imageshiftscale.vmtkImageShiftScale()
    shifter.Image = aorta_image
    shifter.Shift = -2.5
    shifter.Execute()

    assert compare_images(shifter.Image, name) == True


def test_scale_value_positive(aorta_image, compare_images):
    name = __name__ + '_test_scale_value_positive.mha'
    shifter = imageshiftscale.vmtkImageShiftScale()
    shifter.Image = aorta_image
    shifter.Scale = 3.4
    shifter.Execute()

    assert compare_images(shifter.Image, name) == True


def test_scale_value_negative(aorta_image, compare_images):
    name = __name__ + '_test_scale_value_negative.mha'
    shifter = imageshiftscale.vmtkImageShiftScale()
    shifter.Image = aorta_image
    shifter.Scale = -2.1
    shifter.Execute()

    assert compare_images(shifter.Image, name) == True


@pytest.mark.parametrize("outputRange,paramid", [
    ([0.0, 1.0], '0'),
    ([0.0, 359623], '1'),
    ([-87.3, 35963], '2'),
    ([-124, 0.0], '3'),
    ([-123, -34], '4'),
])
def test_map_ranges(aorta_image, compare_images, outputRange, paramid):
    name = __name__ + '_test_map_ranges_' + paramid + '.mha'
    shifter = imageshiftscale.vmtkImageShiftScale()
    shifter.Image = aorta_image
    shifter.MapRanges = 1
    shifter.OutputRange = outputRange
    shifter.Execute()

    assert compare_images(shifter.Image, name) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkbifurcationsections.py,v $
## Language:  Python
## Date:      $Date: 2006/10/17 15:16:16 $
## Version:   $Revision: 1.1 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkBifurcationSections(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Centerlines = None
        self.BifurcationSections = None

        self.NumberOfDistanceSpheres = 1

        self.RadiusArrayName = 'MaximumInscribedSphereRadius'
        self.GroupIdsArrayName = 'GroupIds'
        self.CenterlineIdsArrayName = 'CenterlineIds'
        self.TractIdsArrayName = 'TractIds'
        self.BlankingArrayName = 'Blanking'

        self.OutputSectionPointGroupId = None
        self.OutputSectionPointBifurcationGroupId = None
        self.OutputSectionPoint = []
        self.OutputSectionNormal = []
        self.OutputSectionArea = 0.0

        self.BifurcationSectionGroupIdsArrayName = 'BifurcationSectionGroupIds'
        self.BifurcationSectionBifurcationGroupIdsArrayName = 'BifurcationSectionBifurcationGroupIds'
        self.BifurcationSectionPointArrayName = 'BifurcationSectionPoint'
        self.BifurcationSectionNormalArrayName = 'BifurcationSectionNormal'
        self.BifurcationSectionAreaArrayName = 'BifurcationSectionArea'
        self.BifurcationSectionMinSizeArrayName = 'BifurcationSectionMinSize'
        self.BifurcationSectionMaxSizeArrayName = 'BifurcationSectionMaxSize'
        self.BifurcationSectionShapeArrayName = 'BifurcationSectionShape'
        self.BifurcationSectionClosedArrayName = 'BifurcationSectionClosed'
        self.BifurcationSectionOrientationArrayName = 'BifurcationSectionOrientation'
        self.BifurcationSectionDistanceSpheresArrayName = 'BifurcationSectionDistanceSpheres'

        self.SetScriptName('vmtkbifurcationsections')
        self.SetScriptDoc('compute branch sections located a fixed number of spheres away from each bifurcation. The script takes in input the surface and the relative centerlines, both already split into branches.')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface, already split into branches','vmtksurfacereader'],
            ['Centerlines','centerlines','vtkPolyData',1,'','the input centerlines, already split into branches','vmtksurfacereader'],
            ['NumberOfDistanceSpheres','distancespheres','int',1,'','distance from the bifurcation at which the sections have to be taken; the distance is expressed in number of inscribed spheres, where each sphere touches the center of the previous one'],
            ['RadiusArrayName','radiusarray','str',1,'','name of the array where centerline radius is stored'],
            ['GroupIdsArrayName','groupidsarray','str',1,'','name of the array where centerline group ids are stored'],
            ['CenterlineIdsArrayName','centerlineidsarray','str',1,'','name of the array where centerline ids are stored'],
            ['TractIdsArrayName','tractidsarray','str',1,'','name of the array where centerline tract ids are stored'],
            ['BlankingArrayName','blankingarray','str',1,'','name of the array where centerline blanking information about branches is stored'],
            ['BifurcationSectionGroupIdsArrayName','bifurcationsectiongroupids','str',1,'','name of the array where the group id to which each section belongs has to be stored'],
            ['BifurcationSectionBifurcationGroupIdsArrayName','bifurcationsectionbifurcationgroupids','str',1,'','name of the array where the bifurcation group id to which each section belongs has to be stored'],
            ['BifurcationSectionPointArrayName','bifurcationsectionpoint','str',1,'','name of the array where the point at which bifurcation sections are defined have to be stored'],
            ['BifurcationSectionNormalArrayName','bifurcationsectionnormal','str',1,'','name of the array where the bifurcation section normals have to be stored'],
            ['BifurcationSectionAreaArrayName','bifurcationsectionarea','str',1,'','name of the array where the area of bifurcation sections have to be stored'],
            ['BifurcationSectionMinSizeArrayName','bifurcationsectionminsize','str',1,'','name of the array where the minimum diameter of each section has to be stored'],
            ['BifurcationSectionMaxSizeArrayName','bifurcationsectionmaxsize','str',1,'','name of the array where the maximum diameter of each bifurcation sections has to be stored'],
            ['BifurcationSectionShapeArrayName','bifurcationsectionshape','str',1,'','name of the array where the shape index, i.e. the ratio between minimum and maximum diameter, of each bifurcation section has to be stored'],
            ['BifurcationSectionClosedArrayName','bifurcationsectionclosed','str',1,'','name of the array containing 1 if a section is closed and 0 otherwise'],
            ['BifurcationSectionOrientationArrayName','bifurcationsectionorientation','str',1,'','name of the array containing 0 if a section is upstream and 0 downstream its bifurcation'],
            ['BifurcationSectionDistanceSpheresArrayName','bifurcationsectiondistancespheres','str',1,'','name of the array containing the number of spheres away from the bifurcation the section is located at'],
            ['OutputSectionPointGroupId','sectionpointgroupid','int',1,'(0,)'],
            ['OutputSectionPointBifurcationGroupId','sectionpointbifurcationgroupid','int',1,'(0,)']
            ])
        self.SetOutputMembers([
            ['BifurcationSections','o','vtkPolyData',1,'','the output sections','vmtksurfacewriter'],
            ['BifurcationSectionGroupIdsArrayName','bifurcationsectiongroupids','str',1,'','name of the array where the group id to which each section belongs are stored'],
            ['BifurcationSectionBifurcationGroupIdsArrayName','bifurcationsectionbifurcationgroupids','str',1,'','name of the array where the bifurcation group id to which each section belongs has to be stored'],
            ['BifurcationSectionPointArrayName','bifurcationsectionpoint','str',1,'','name of the array where the point at which bifurcation sections are defined are stored'],
            ['BifurcationSectionNormalArrayName','bifurcationsectionnormal','str',1,'','name of the array where bifurcation section normals are stored'],
            ['BifurcationSectionAreaArrayName','bifurcationsectionarea','str',1,'','name of the array where the area of bifurcation sections are stored'],
            ['BifurcationSectionMinSizeArrayName','bifurcationsectionminsize','str',1,'','name of the array where the minimum diameter of each section are stored'],
            ['BifurcationSectionMaxSizeArrayName','bifurcationsectionmaxsize','str',1,'','name of the array where the minimum diameter of each bifurcation sections has to be stored'],
            ['BifurcationSectionShapeArrayName','bifurcationsectionshape','str',1,'','name of the array where the shape index, i.e. the ratio between minimum and maximum diameter, of each bifurcation section are stored'],
            ['BifurcationSectionClosedArrayName','bifurcationsectionclosed','str',1,'','name of the array containing 1 if a section is closed and 0 otherwise'],
            ['BifurcationSectionOrientationArrayName','bifurcationsectionorientation','str',1,'','name of the array containing 0 if a section is upstream and 0 downstream its bifurcation'],
            ['BifurcationSectionDistanceSpheresArrayName','bifurcationsectiondistancespheres','str',1,'','name of the array containing the number of spheres away from the bifurcation the section is located at'],
            ['OutputSectionPoint','sectionpoint','float',3],
            ['OutputSectionNormal','sectionnormal','float',3],
            ['OutputSectionArea','sectionarea','float',1]
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        bifurcationSections = vtkvmtk.vtkvmtkPolyDataBifurcationSections()
        bifurcationSections.SetInputData(self.Surface)
        bifurcationSections.SetGroupIdsArrayName(self.GroupIdsArrayName)
        bifurcationSections.SetCenterlines(self.Centerlines)
        bifurcationSections.SetNumberOfDistanceSpheres(self.NumberOfDistanceSpheres)
        bifurcationSections.SetCenterlineRadiusArrayName(self.RadiusArrayName)
        bifurcationSections.SetCenterlineGroupIdsArrayName(self.GroupIdsArrayName)
        bifurcationSections.SetCenterlineIdsArrayName(self.CenterlineIdsArrayName)
        bifurcationSections.SetCenterlineTractIdsArrayName(self.TractIdsArrayName)
        bifurcationSections.SetBlankingArrayName(self.BlankingArrayName)
        bifurcationSections.SetBifurcationSectionGroupIdsArrayName(self.BifurcationSectionGroupIdsArrayName)
        bifurcationSections.SetBifurcationSectionBifurcationGroupIdsArrayName(self.BifurcationSectionBifurcationGroupIdsArrayName)
        bifurcationSections.SetBifurcationSectionPointArrayName(self.BifurcationSectionPointArrayName)
        bifurcationSections.SetBifurcationSectionNormalArrayName(self.BifurcationSectionNormalArrayName)
        bifurcationSections.SetBifurcationSectionAreaArrayName(self.BifurcationSectionAreaArrayName)
        bifurcationSections.SetBifurcationSectionMinSizeArrayName(self.BifurcationSectionMinSizeArrayName)
        bifurcationSections.SetBifurcationSectionMaxSizeArrayName(self.BifurcationSectionMaxSizeArrayName)
        bifurcationSections.SetBifurcationSectionShapeArrayName(self.BifurcationSectionShapeArrayName)
        bifurcationSections.SetBifurcationSectionClosedArrayName(self.BifurcationSectionClosedArrayName)
        bifurcationSections.SetBifurcationSectionOrientationArrayName(self.BifurcationSectionOrientationArrayName)
        bifurcationSections.SetBifurcationSectionDistanceSpheresArrayName(self.BifurcationSectionDistanceSpheresArrayName)
        bifurcationSections.Update()

        self.BifurcationSections = bifurcationSections.GetOutput()

        if self.OutputSectionPointGroupId != None and self.OutputSectionPointBifurcationGroupId != None:
            groupIds = self.BifurcationSections.GetCellData().GetArray(self.BifurcationSectionGroupIdsArrayName)
            bifurcationGroupIds = self.BifurcationSections.GetCellData().GetArray(self.BifurcationSectionBifurcationGroupIdsArrayName)
            for i in range(self.BifurcationSections.GetNumberOfCells()):
                if int(groupIds.GetTuple1(i)) == self.OutputSectionPointGroupId and int(bifurcationGroupIds.GetTuple1(i)) == self.OutputSectionPointBifurcationGroupId:
                    self.OutputSectionPoint = self.BifurcationSections.GetCellData().GetArray(self.BifurcationSectionPointArrayName).GetTuple3(i)
                    self.OutputSectionNormal = self.BifurcationSections.GetCellData().GetArray(self.BifurcationSectionNormalArrayName).GetTuple3(i)
                    self.OutputSectionArea = self.BifurcationSections.GetCellData().GetArray(self.BifurcationSectionAreaArrayName).GetTuple1(i)
                    self.InputInfo('SectionPoint: '+str(self.OutputSectionPoint))
                    self.InputInfo('SectionNormal: '+str(self.OutputSectionNormal))
                    self.InputInfo('SectionArea: '+str(self.OutputSectionArea))


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtksurfacemodeller as modeller


def test_default_params(aorta_surface, compare_images):
    name = __name__ + '_test_default_params.mha'
    model = modeller.vmtkSurfaceModeller()
    model.Surface = aorta_surface
    model.Execute()

    assert compare_images(model.Image, name) == True


def test_turn_off_negative_inside(aorta_surface, compare_images):
    name = __name__ + '_test_negative_inside.mha'
    model = modeller.vmtkSurfaceModeller()
    model.Surface = aorta_surface
    model.NegativeInside = 0
    model.Execute()

    assert compare_images(model.Image, name) == True


def test_change_sample_spacing(aorta_surface, compare_images):
    name = __name__ + '_test_change_sample_spacing.mha'
    model = modeller.vmtkSurfaceModeller()
    model.Surface = aorta_surface
    model.SampleSpacing = -0.5
    model.Execute()

    assert compare_images(model.Image, name) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagereader.py,v $
## Language:  Python
## Date:      $Date: 2006/05/22 08:33:12 $
## Version:   $Revision: 1.16 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkImageReader(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Format = ''
        self.GuessFormat = 1
        self.UseITKIO = 1
        self.InputFileName = ''
        self.InputFilePrefix = ''
        self.InputFilePattern = ''
        self.Image = 0
        self.Output = 0

        self.DataExtent = [-1, -1, -1, -1, -1, -1]
        self.DataSpacing = [1.0, 1.0, 1.0]
        self.DataOrigin = [0.0, 0.0, 0.0]
        self.DataByteOrder = 'littleendian'
        self.DataScalarType = 'float'
        self.DesiredOrientation = 'native'
        self.HeaderSize = 0
        self.FileDimensionality = 3
        self.Flip = [0, 0, 0]
        self.AutoOrientDICOMImage = 1
        self.RasToIjkMatrixCoefficients = [1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1]
        self.XyzToRasMatrixCoefficients = [1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1]

        self.SetScriptName('vmtkimagereader')
        self.SetScriptDoc('read an image and stores it in a vtkImageData object')
        self.SetInputMembers([
            ['Format','f','str',1,'["vtkxml","vtk","dicom","raw","meta","tiff","png"]','file format'],
            ['GuessFormat','guessformat','bool',1,'','guess file format from extension'],
            ['UseITKIO','useitk','bool',1,'','use ITKIO mechanism'],
            ['Image','i','vtkImageData',1,'','the input image'],
            ['InputFileName','ifile','str',1,'','input file name'],
            ['InputFilePrefix','prefix','str',1,'','input file prefix (e.g. foo_)'],
            ['InputFilePattern','pattern','str',1,'','input file pattern (e.g. %s%04d.png)'],
            ['DataExtent','extent','int',6,'','3D extent of the image - raw and png'],
            ['HeaderSize','headersize','int',1,'(0,)','size of the image header - raw only'],
            ['DataSpacing','spacing','float',3,'','spacing of the image - raw, tiff, png, itk'],
            ['DataOrigin','origin','float',3,'','origin of the image - raw, tiff, png, itk'],
            ['DesiredOrientation','orientation','str',1,'["native","axial","coronal","sagittal"]','desired data orientation - itk only'],
            ['DataByteOrder','byteorder','str',1,'["littleendian","bigendian"]','byte ordering - raw only'],
            ['DataScalarType','scalartype','str',1,'["float","double","int","short","ushort","uchar"]','scalar type - raw only'],
            ['FileDimensionality','filedimensionality','int',1,'(2,3)','dimensionality of the file to read - raw only'],
            ['Flip','flip','bool',3,'','toggle flipping of the corresponding axis'],
            ['AutoOrientDICOMImage','autoorientdicom','bool',1,'','flip a dicom stack in order to have a left-to-right, posterio-to-anterior, inferior-to-superior image; this is based on the \"image orientation (patient)\" field in the dicom header']
            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter'],
            ['RasToIjkMatrixCoefficients','rastoijkmatrix','float',16],
            ['XyzToRasMatrixCoefficients','xyztorasmatrix','float',16]
            ])

    def ReadVTKXMLImageFile(self):
        if (self.InputFileName == ''):
            self.PrintError('Error: no InputFileName.')
        self.PrintLog('Reading VTK XML image file.')
        reader = vtk.vtkXMLImageDataReader()
        reader.SetFileName(self.InputFileName)
        reader.Update()
        self.Image = reader.GetOutput()

    def ReadVTKImageFile(self):
        if (self.InputFileName == ''):
            self.PrintError('Error: no InputFileName.')
        self.PrintLog('Reading VTK image file.')
        reader = vtk.vtkStructuredPointsReader()
        reader.SetFileName(self.InputFileName)
        reader.Update()
        self.Image = reader.GetOutput()

    def ReadRawImageFile(self):
        if (self.InputFileName == '') & (self.InputFilePrefix == ''):
            self.PrintError('Error: no InputFileName or InputFilePrefix.')
        self.PrintLog('Reading RAW image file.')
        reader = vtk.vtkImageReader()
        if self.InputFileName != '':
            reader.SetFileName(self.InputFileName)
        else:
            reader.SetFilePrefix(self.InputFilePrefix)
            if self.InputFilePattern != '':
                reader.SetFilePattern(self.InputFilePattern)
            else:
                reader.SetFilePattern("%s%04d.png")
        reader.SetFileDimensionality(self.FileDimensionality)
        if self.DataByteOrder == 'littleendian':
            reader.SetDataByteOrderToLittleEndian()
        elif self.DataByteOrder == 'bigendian':
            reader.SetDataByteOrderToBigEndian()
        reader.SetDataExtent(self.DataExtent)
        reader.SetDataSpacing(self.DataSpacing)
        reader.SetDataOrigin(self.DataOrigin)
        reader.SetHeaderSize(self.HeaderSize)
        if self.DataScalarType == 'float':
            reader.SetDataScalarTypeToFloat()
        elif self.DataScalarType == 'double':
            reader.SetDataScalarTypeToDouble()
        elif self.DataScalarType == 'int':
            reader.SetDataScalarTypeToInt()
        elif self.DataScalarType == 'short':
            reader.SetDataScalarTypeToShort()
        elif self.DataScalarType == 'ushort':
            reader.SetDataScalarTypeToUnsignedShort()
        elif self.DataScalarType == 'uchar':
            reader.SetDataScalarTypeToUnsignedChar()
        reader.Update()
        self.Image = reader.GetOutput()

    def ReadMetaImageFile(self):
        if (self.InputFileName == ''):
            self.PrintError('Error: no InputFileName.')
        self.PrintLog('Reading meta image file.')
        reader = vtk.vtkMetaImageReader()
        reader.SetFileName(self.InputFileName)
        reader.Update()
        self.Image = reader.GetOutput()

    def ReadTIFFImageFile(self):
        if (self.InputFileName == '') & (self.InputFilePrefix == ''):
            self.PrintError('Error: no InputFileName or InputFilePrefix.')
        self.PrintLog('Reading TIFF image file.')
        reader = vtk.vtkTIFFReader()
        if self.InputFileName != '':
            reader.SetFileName(self.InputFileName)
        else:
            reader.SetFilePrefix(self.InputFilePrefix)
            if self.InputFilePattern != '':
                reader.SetFilePattern(self.InputFilePattern)
            else:
                reader.SetFilePattern("%s%04d.png")
            reader.SetDataExtent(self.DataExtent)
            reader.SetDataSpacing(self.DataSpacing)
            reader.SetDataOrigin(self.DataOrigin)
        reader.Update()
        self.Image = reader.GetOutput()

    def ReadPNGImageFile(self):
        if (self.InputFileName == '') & (self.InputFilePrefix == ''):
            self.PrintError('Error: no InputFileName or InputFilePrefix.')
        self.PrintLog('Reading PNG image file.')
        reader = vtk.vtkPNGReader()
        if self.InputFileName != '':
            reader.SetFileName(self.InputFileName)
        else:
            reader.SetFilePrefix(self.InputFilePrefix)
            if self.InputFilePattern != '':
                reader.SetFilePattern(self.InputFilePattern)
            else:
                reader.SetFilePattern("%s%04d.png")
            reader.SetDataExtent(self.DataExtent)
            reader.SetDataSpacing(self.DataSpacing)
            reader.SetDataOrigin(self.DataOrigin)
        reader.Update()
        self.Image = reader.GetOutput()

    def ReadITKIO(self):
        if self.InputFileName == '':
            self.PrintError('Error: no InputFileName.')
        reader = vtkvmtk.vtkvmtkITKArchetypeImageSeriesScalarReader()
        reader.SetArchetype(self.InputFileName)
        reader.SetDefaultDataSpacing(self.DataSpacing)
        reader.SetDefaultDataOrigin(self.DataOrigin)
        reader.SetOutputScalarTypeToNative()
        if self.DesiredOrientation == 'native':
            reader.SetDesiredCoordinateOrientationToNative()
        elif self.DesiredOrientation == 'axial':
            reader.SetDesiredCoordinateOrientationToAxial()
        elif self.DesiredOrientation == 'coronal':
            reader.SetDesiredCoordinateOrientationToCoronal()
        elif self.DesiredOrientation == 'sagittal':
            reader.SetDesiredCoordinateOrientationToSagittal()
        reader.SetSingleFile(0)
        reader.Update()
        self.Image = vtk.vtkImageData()
        self.Image.DeepCopy(reader.GetOutput())
        matrix = reader.GetRasToIjkMatrix()
        self.RasToIjkMatrixCoefficients = [
            matrix.GetElement(0,0), matrix.GetElement(0,1), matrix.GetElement(0,2), matrix.GetElement(0,3),
            matrix.GetElement(1,0), matrix.GetElement(1,1), matrix.GetElement(1,2), matrix.GetElement(1,3),
            matrix.GetElement(2,0), matrix.GetElement(2,1), matrix.GetElement(2,2), matrix.GetElement(2,3),
            matrix.GetElement(3,0), matrix.GetElement(3,1), matrix.GetElement(3,2), matrix.GetElement(3,3)]

        matrix.Invert()
        origin = [matrix.GetElement(0,3), matrix.GetElement(1,3), matrix.GetElement(2,3)]
        translationToOrigin = [-origin[0], -origin[1], -origin[2]]

        for i in range(3):
            direction = [matrix.GetElement(0,i), matrix.GetElement(1,i), matrix.GetElement(2,i)]
            vtk.vtkMath.Normalize(direction)
            matrix.SetElement(0,i,direction[0])
            matrix.SetElement(1,i,direction[1])
            matrix.SetElement(2,i,direction[2])
        matrix.SetElement(0,3,0.0)
        matrix.SetElement(1,3,0.0)
        matrix.SetElement(2,3,0.0)

        transform = vtk.vtkTransform()
        transform.PostMultiply()
        transform.Translate(translationToOrigin)
        transform.Concatenate(matrix)
        transform.Translate(origin)

        matrix = transform.GetMatrix()
        self.XyzToRasMatrixCoefficients = [
            matrix.GetElement(0,0), matrix.GetElement(0,1), matrix.GetElement(0,2), matrix.GetElement(0,3),
            matrix.GetElement(1,0), matrix.GetElement(1,1), matrix.GetElement(1,2), matrix.GetElement(1,3),
            matrix.GetElement(2,0), matrix.GetElement(2,1), matrix.GetElement(2,2), matrix.GetElement(2,3),
            matrix.GetElement(3,0), matrix.GetElement(3,1), matrix.GetElement(3,2), matrix.GetElement(3,3)]

    def Execute(self):

        extensionFormats = {'vti':'vtkxml',
                            'vtkxml':'vtkxml',
                            'vtk':'vtk',
                            'dcm':'dicom',
                            'raw':'raw',
                            'mhd':'meta',
                            'mha':'meta',
                            'tif':'tiff',
                            'png':'png'}

        if self.InputFileName == 'BROWSER':
            import tkinter.filedialog
            import os.path
            initialDir = pypes.pypeScript.lastVisitedPath
            self.InputFileName = tkinter.filedialog.askopenfilename(title="Input image",initialdir=initialDir)
            pypes.pypeScript.lastVisitedPath = os.path.dirname(self.InputFileName)
            if not self.InputFileName:
                self.PrintError('Error: no InputFileName.')

        if self.GuessFormat and self.InputFileName and not self.Format:
            import os.path
            extension = os.path.splitext(self.InputFileName)[1]
            if extension:
                extension = extension[1:]
                if extension in list(extensionFormats.keys()):
                    self.Format = extensionFormats[extension]

        if self.UseITKIO and self.InputFileName and self.Format not in ['vtkxml','vtk','raw']:
            self.ReadITKIO()
        else:
            if self.Format == 'vtkxml':
                self.ReadVTKXMLImageFile()
            elif self.Format == 'vtk':
                self.ReadVTKImageFile()
            elif self.Format == 'raw':
                self.ReadRawImageFile()
            elif self.Format == 'meta':
                self.ReadMetaImageFile()
            elif self.Format == 'png':
                self.ReadPNGImageFile()
            elif self.Format == 'tiff':
                self.ReadTIFFImageFile()
            elif self.Format == 'dicom':
                self.PrintError('Error: please enable parameter UseITKIO in order to read dicom files')
            else:
                self.PrintError('Error: unsupported format '+ self.Format + '.')

        if (self.Flip[0] == 1) | (self.Flip[1] == 1) | (self.Flip[2] == 1):
            temp0 = self.Image
            if self.Flip[0] == 1:
                flipFilter = vtk.vtkImageFlip()
                flipFilter.SetInputData(self.Image)
                flipFilter.SetFilteredAxis(0)
                flipFilter.Update()
                temp0 = flipFilter.GetOutput()
            temp1 = temp0
            if self.Flip[1] == 1:
                flipFilter = vtk.vtkImageFlip()
                flipFilter.SetInputData(temp0)
                flipFilter.SetFilteredAxis(1)
                flipFilter.Update()
                temp1 = flipFilter.GetOutput()
            temp2 = temp1
            if self.Flip[2] == 1:
                flipFilter = vtk.vtkImageFlip()
                flipFilter.SetInputData(temp1)
                flipFilter.SetFilteredAxis(2)
                flipFilter.Update()
                temp2 = flipFilter.GetOutput()
            self.Image = temp2

        self.PrintLog('Spacing %f %f %f' % self.Image.GetSpacing())
        self.PrintLog('Origin %f %f %f' % self.Image.GetOrigin())
        self.PrintLog('Dimensions %d %d %d' % self.Image.GetDimensions())

        self.Output = self.Image


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()
